''*****************************************************************************
''
''
''	Asteroid class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Asteroid.bi"


''*****************************************************************************
''
'' Asteroid
''
''*****************************************************************************
constructor Asteroid()

	Active = FALSE	
	Hp = ASTEROID_MAX_HP 
	DrawMode = DRAW_NORMAL
	HitTimer = 0
	
	Body.Create( 4, 30, 0.7 )
	
	Position = Vector2D( -SCREEN_WIDTH, -SCREEN_HEIGHT )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Constructor

destructor Asteroid()

End Destructor

''*****************************************************************************
''
''
''
''*****************************************************************************
property Asteroid.IsActive() as integer
	property = Active
End Property

property Asteroid.GetPoly() as Polygon
	property = Body
End Property

property Asteroid.GetHp() as integer
	property = Hp
End Property

property Asteroid.GetRadius() as integer
	property = Radius
End Property

property Asteroid.GetPosition() as Vector2D 
	property = Position
End Property

property Asteroid.DecHp( byval value as integer )
	Hp -= value
End Property

property Asteroid.SetDrawMode( byval value as integer )
	DrawMode = value
End Property
	
''*****************************************************************************
''
''
''
''*****************************************************************************
sub Asteroid.Init( byref Posi as Vector2D, byref Target as Vector2D,_ 
		   		   byval spd as single, byval Col as integer,_ 
		   		   byval NumVerts as integer, byval rads as single,_
		   		   byval ScaleX as single = 1.0f, byval ScaleY as single = 1.0f )

	Active = TRUE
	Colour = Col
	Speed = spd	
	Delta = -3.0f + rnd * 6.0f
	Radius = rads
	Hp = Radius/2 
	
	Position = Posi
	
	Angle = UtilVector2D.AngleBetween( Posi, Target )
	Direction = Vector2D( cos(Angle) * Speed, sin(Angle) * Speed )
	
	Body.Create( NumVerts, Radius, ScaleX, ScaleY )
	
end sub		   		

sub Asteroid.Update( byref Fighter as ship, byval dt as single )
	
	Position += Direction * dt
	
	if( (Position.x < -100) or (Position.x > (SCREEN_WIDTH + 100)) or _
		(Position.y < -100) or (Position.y > (SCREEN_HEIGHT + 100)) ) then
		Kill()
	endif
	
	Angle += Delta * dt
		
	Body.Reset()
	Body.Rotate( RAD2DEG(Angle) ) 
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
	if( DrawMode = DRAW_WHITE ) then
		HitTimer += 1.0f * dt
		if( HitTimer > 0.01f ) then	'' Blink for 0.01 of a second
			DrawMode = DRAW_NORMAL
			HitTimer = 0
		endif
	endif
	
End Sub

sub Asteroid.Draw()
	
	if( DrawMode = DRAW_WHITE ) then
		for i as integer = 0 to 2
			Body.Draw( 20 - i, GL2D_RGBA( 255, 255, 255, 255 ) )
		next i
	else
		for i as integer = 0 to 1
			Body.Draw( 10 - i, Colour )
		next i
	endif
		
End Sub

''*****************************************************************************
''
''
''
''*****************************************************************************
sub Asteroid.Destroy()
	
	Explosion.Spawn(Position, 64 )		
	Kill()
	
End Sub

sub Asteroid.Kill()	
	
	Active = FALSE
	Position = Vector2D( -3600, -3600 )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Sub
	
''*****************************************************************************
''
'' AsteroidFactory
''
''*****************************************************************************

constructor AsteroidFactory()

End Constructor

destructor AsteroidFactory()

End Destructor

property AsteroidFactory.GetNumEntities() as Integer
	property = NumEntities
end property 

sub AsteroidFactory.UpdateEntities( byref Fighter as Ship, byval dt as single )
	
	NumEntities = 0
	for i as integer = 0 to MAX_ASTEROIDS
		if( Asteroids(i).IsActive ) then
			Asteroids(i).Update( Fighter, dt )
			NumEntities += 1
		EndIf
	Next
	
end sub

sub AsteroidFactory.DrawEntities()
	
	for i as integer = 0 to MAX_ASTEROIDS
		if( Asteroids(i).IsActive ) then
			Asteroids(i).Draw()
		EndIf
	Next
	
end sub

sub AsteroidFactory.KillAllEntities()
	
	for i as integer = 0 to MAX_ASTEROIDS
		if( Asteroids(i).IsActive ) then
			Asteroids(i).Kill()
		EndIf
	Next
	
end sub

sub AsteroidFactory.HandleCollisions( byref Fighter as Ship )
	
	for i as integer = 0 to MAX_ASTEROIDS
		
		if( Asteroids(i).IsActive ) then
			
			'' Ship to Asteroid collisions
			if( Fighter.HandleCollision( Asteroids(i).Body) ) then
				if( Fighter.IsInvincible = FALSE ) then
					Fighter.Destroy()
				endif
			end if
			
			
			'' Ship shots to Asteroid collisions
			dim as integer Collision = Fighter.HandleShotCollisions( Asteroids(i).Body )
			if( Collision ) then
				
				Asteroids(i).SetDrawMode = DRAW_WHITE
				Asteroids(i).DecHp =  Collision 
				
				if( Asteroids(i).GetHp < 0 ) then
					
					'' Spawn New asteroid if asteroid is big enough
					dim as integer NumSpawns = Asteroids(i).GetRadius \ 10
					if( NumSpawns > 1 ) then
					
						for j as integer = 0 to NumSpawns - 1 
							dim as integer NumVerts = 3 + rnd * 8
							dim as single Radius = Asteroids(i).GetRadius/2
							if( Radius < 10 ) then Radius = 10
							dim as single ScaleX = 0.3f + rnd * 1.0f
							dim as single Speed = 20 + rnd * 80
							dim as integer Colour = GL2D_RGBA( 127 + rnd * 255, 127 + rnd * 255, 127 + rnd * 255, 255 )
							dim as single Angle = rnd * (2.0f * PI )
							dim as integer Vr = 350
							dim as Vector2D Posi = Asteroids(i).GetPosition
							dim as Vector2D Target = Vector2D( Posi.x + cos(Angle) * Vr,_
															   Posi.y + sin(Angle) * Vr )
				
							Spawn( Posi, Target,_
								   Speed, Colour,_
								   NumVerts, Radius, ScaleX, 1.0f )
						next j
						
						Sound.PlaySFX( Sound.SFX_EXPLODE_03 )
					else
						Sound.PlaySFX( Sound.SFX_EXPLODE_01 )
					endif
					
					
					'' Increase score
					Fighter.AddToScore( Asteroids(i).GetRadius * 3 )
					'' destroy original
					Asteroids(i).Destroy()
					
				endif
			end if
			
		EndIf
		
	Next i
	
end sub

sub AsteroidFactory.Spawn( byref Posi as Vector2D, byref Target as Vector2D,_ 
		   		   		   byval spd as single, byval Col as integer,_ 
				   		   byval NumVerts as integer, byval radius as single,_
				   		   byval ScaleX as single = 1.0f, byval ScaleY as single = 1.0f )

	
	for i as integer = 0 to MAX_ASTEROIDS
		if( Asteroids(i).IsActive = FALSE ) then
			Asteroids(i).Init( Posi, Target, spd, Col, NumVerts, Radius, ScaleX, ScaleY )
			exit for
		EndIf
	Next
	
end sub
