''*****************************************************************************
''
''
''	Bonus class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Bonus.bi"


''*****************************************************************************
''
'' Bonus
''
''*****************************************************************************
constructor Bonus()

	Active = FALSE	
	
	Body.Create( 8, 20 )
	
	Position = Vector2D( -SCREEN_WIDTH, -SCREEN_HEIGHT )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
	
End Constructor

destructor Bonus()

End Destructor

''*****************************************************************************
''
''
''
''*****************************************************************************
property Bonus.IsActive() as integer
	property = Active
End Property

property Bonus.GetID() as integer
	property = ID
End Property

property Bonus.GetPoly() as Polygon
	property = Body
End Property
	
''*****************************************************************************
''
''
''
''*****************************************************************************
sub Bonus.Init( byref Posi as Vector2D )

	ID = int(rnd * 3)
	Active = TRUE
	Speed = ( 100.0f + rnd * 200.0f )
	Angle = rnd * ( 2 * PI )
	Delta = 8.0f
	Intensity = 1.0f
	Direction = Vector2D( cos(Angle) * Speed, sin(Angle) * Speed )
	
	Position = Posi
	select case ID
		case 0:
			Colour = GL2D_RGBA( 255,   0,   0, 255 )
		case 1:
			Colour = GL2D_RGBA(   0, 255,   0, 255 )
		case 2:
			Colour = GL2D_RGBA(   0,   0, 255, 255 )
		case else
			Colour = GL2D_RGBA( 255,   0,   0, 255 )
	end select
end sub		   		

sub Bonus.Update( byref Fighter as ship, byval dt as single )
	
	Position += Direction * dt
	
	if( Position.x < 0 ) then 
		Position.x = 0
		Direction.x = -Direction.x
	EndIf
	if( Position.x > SCREEN_WIDTH ) then 
		Position.x = SCREEN_WIDTH
		Direction.x = -Direction.x
	EndIf
	if( Position.y < 0 ) then 
		Position.y = 0
		Direction.y = -Direction.y
	EndIf
	if( Position.y > SCREEN_HEIGHT ) then
		Position.y = SCREEN_HEIGHT
		Direction.y = -Direction.y
	EndIf
	
	Angle += Delta * dt
	Intensity = 5 + abs(sin((Angle/5.0f))) * 15
	Body.Reset()
	Body.Rotate( RAD2DEG(Angle) ) 
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Sub

sub Bonus.Draw()
	
	for i as integer = 0 to 2
		Body.Draw( 8 + (i * Intensity), Colour )
	next i
		
End Sub

''*****************************************************************************
''
''
''
''*****************************************************************************
sub Bonus.Destroy()

	Kill()
	
End Sub

sub Bonus.Kill()	
	
	Active = FALSE
	Position = Vector2D( -3600, -3600 )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Sub
	
''*****************************************************************************
''
'' BonusFactory
''
''*****************************************************************************

constructor BonusFactory()

End Constructor

destructor BonusFactory()

End Destructor

property BonusFactory.GetNumEntities() as Integer
	property = NumEntities
end property 

sub BonusFactory.UpdateEntities( byref Fighter as Ship, byval dt as single )
	
	NumEntities = 0
	for i as integer = 0 to MAX_BONUSES
		if( Bonuses(i).IsActive ) then
			Bonuses(i).Update( Fighter, dt )
			NumEntities += 1
		EndIf
	Next
	
end sub

sub BonusFactory.DrawEntities()
	
	for i as integer = 0 to MAX_BONUSES
		if( Bonuses(i).IsActive ) then
			Bonuses(i).Draw()
		EndIf
	Next
	
end sub

sub BonusFactory.KillAllEntities()
	
	for i as integer = 0 to MAX_BONUSES
		if( Bonuses(i).IsActive ) then
			Bonuses(i).Kill()
		EndIf
	Next
	
end sub

sub BonusFactory.HandleCollisions( byref Fighter as Ship )
	
	for i as integer = 0 to MAX_BONUSES
		
		if( Bonuses(i).IsActive ) then
			'' Ship to Bonus collisions
			if( Fighter.HandleCollision( Bonuses(i).Body) ) then
				Fighter.AddToScore( 500 * ( Bonuses(i).GetID + 1) )
				Bonuses(i).Destroy()	
				Sound.PlaySFX( Sound.SFX_POWER_UP )
			end if
		EndIf
		
	Next
	
end sub

sub BonusFactory.Spawn( byref Posi as Vector2D )

	
	for i as integer = 0 to MAX_BONUSES
		if( Bonuses(i).IsActive = FALSE ) then
			Bonuses(i).Init( Posi )
			exit for
		EndIf
	Next
	
end sub
