''*****************************************************************************
''
''
''	Engine class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Engine.bi"

#define MAX_RADIUS 720      '' Radius of the web

''  Since I only use 2 buttons, I'll just remap the joykeys to Ok and Cancel
''  Call me lazy 'cause I am.
const as integer JOY_FIRE = JOY_KEY_1 or JOY_KEY_2 
const as integer JOY_OK = JOY_KEY_3 or JOY_KEY_5 or JOY_KEY_7 or JOY_KEY_9 or JOY_KEY_11 or JOY_KEY_13 or JOY_KEY_15
const as integer JOY_CANCEL = JOY_KEY_4 or JOY_KEY_6 or JOY_KEY_8 or JOY_KEY_10 or JOY_KEY_12 or JOY_KEY_14 or JOY_KEY_16

''  Difficulty level
const as single ENEMY_SPAWN_DELAY = 3.0
 

''  Utility function to interpolate from a to b given t[0..1]  
private function Lerp( byval a as single, byval b as single, byval t as single ) as single
	return a + (b-a) * t
end function
''*****************************************************************************
''
''
''
''*****************************************************************************
constructor Engine()
	
	State = STATE_TITLE
	Level = 0
	Frame = 0
	SecondsElapsed = 0
	FPS = 60
	dt = 0
	GeoSpawnDelay = 0
	MaxGeoSpawnDelay = ENEMY_SPAWN_DELAY
		
	PowSpawnDelay = 10
	MaxPowSpawnDelay = 10.0
	
	BonusSpawnDelay = 17
	MaxBonusSpawnDelay = 17.0
	
	'' Init BG
	for i as integer = 0 to MAX_RINGS
		T(i).x = 320
		T(i).y = 240
		T(i).r = Lerp( 0, MAX_RADIUS , i / (MAX_RINGS + 1) )
	Next

	MouseTarget.Create( 14, 20 )
	
	ControlMode = Ship.MODE_NORMAL
	FullScreen = FALSE
	Vsynch = FALSE
	CapFPS = FALSE
	ShowFPS = TRUE
	AutoFire = TRUE
	
	MasterVolumeBGM = 128
	MasterVolumeSFX = 255
	
End Constructor

destructor Engine()

End Destructor

''*****************************************************************************
''
''
''
''*****************************************************************************
property Engine.SetState( byval v as integer )
	State = v
End Property

''*****************************************************************************
''
''
''
''*****************************************************************************
sub Engine.Initialize()
	
	State = STATE_TITLE
	Level = 0
	Frame = 0
	SecondsElapsed = 0
	FPS = 60
	dt = 0
	GeoSpawnDelay = 0
	MaxGeoSpawnDelay = ENEMY_SPAWN_DELAY
	
End Sub

function Engine.Update() as integer
	
	''  Check-up what function to call depending on the state of the engine
	dim as integer Done = FALSE
	
	select case State		
		case STATE_PLAY:
			Done = StatePlay()
		case STATE_PAUSE:
			Done = StatePause()
		case STATE_START:
			Done = StateStart()
		case STATE_GAME_OVER:
			Done = StateGameOver()
		case STATE_OPTIONS:
			Done = StateOptions()
		case STATE_EXTRAS:
			Done = StateExtras()
		case STATE_CREDITS:
			Done = StateCredits()
		case STATE_TITLE:
			Done = StateTitle()
		case STATE_YES_OR_NO:
			Done = StateYesOrNo()
		case STATE_EXIT:
			
	End Select
	
	return Done
	
End function


sub Engine.InitEverything()
	
	randomize timer

	Sound.Initialize( 44000, 32, 0 )

	Fighter.Initialize()
	
	Sound.SetMasterVolumeBGM( MasterVolumeBGM )
	Sound.SetMasterVolumeSFX( MasterVolumeSFX )
 	
 	'' Load config file if there is one
	if( dir( "ConvexWars.cfg" ) = "ConvexWars.cfg" ) then
		LoadConfig( "ConvexWars.cfg" )	
	EndIf
	
	
	Fighter.SetControlMode = ControlMode
	Fighter.SetAutoFire = AutoFire
	
	dim as integer flag = FB.GFX_WINDOWED
	if( FullScreen ) then flag = FB.GFX_FULLSCREEN
	
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT, flag or FB.GFX_NO_FRAME )   ''Set up GL screen
	
	GL2D.EnableAntialias( TRUE )
	
	if( Vsynch ) then
		GL2D.VsyncOn()
	EndIf
	
	
	GL2D.ClearScreen()
	
	GL2D.Begin2D()
	
		glColor4ub(255,255,0,255)
		Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
	    
	    dim as string sig = __FB_SIGNATURE__
	    CenterText( 0, 1 ,"CONVEX WARS FB", 8 )
		glColor4ub(0,255,0,255)
		CenterText( 20, 1 ,"By Richard Eric M. Lope", 8 )
		CenterText( 30, 1 ,"Relminator", 8 )
		CenterText( 50, 1 ,"Compiled with " & sig, 8 )
		
	GL2D.End2D()
	
	sleep 1,1
	
	flip
	
	glColor4ub(255,255,255,255)	
	PrintLog( 0, 100, 1, "Engine Initialize"  )
	
End Sub

sub Engine.LoadSounds()
	
	PrintLog( 0, 120, 1, "Loading Sound Effects..." )
	
	Sound.LoadSFX( "sound/sfx/shot.wav", Sound.SFX_SHOT )
	Sound.LoadSFX( "sound/sfx/enter.wav", Sound.SFX_ENTER )
	Sound.LoadSFX( "sound/sfx/pause.wav", Sound.SFX_PAUSE )
	Sound.LoadSFX( "sound/sfx/enemyhit.wav", Sound.SFX_ENEMY_HIT )
	Sound.LoadSFX( "sound/sfx/explode01.wav", Sound.SFX_EXPLODE_01 )
	Sound.LoadSFX( "sound/sfx/explode02.wav", Sound.SFX_EXPLODE_02 )
	Sound.LoadSFX( "sound/sfx/explode03.wav", Sound.SFX_EXPLODE_03 )
	Sound.LoadSFX( "sound/sfx/click.wav", Sound.SFX_CLICK )
	Sound.LoadSFX( "sound/sfx/powerup.wav", Sound.SFX_POWER_UP )
	
	
	PrintLog( 0, 140, 1, "Loading Music..." )
	
	Sound.LoadBGM( "sound/bgm/level_01.mid", Sound.BGM_LEVEL_01, 128 )
	Sound.LoadBGM( "sound/bgm/level_02.mid", Sound.BGM_LEVEL_02, 128 )
	Sound.LoadBGM( "sound/bgm/level_03.mid", Sound.BGM_LEVEL_03, 128 )
	Sound.LoadBGM( "sound/bgm/level_boss.mid", Sound.BGM_LEVEL_BOSS, 128 )
	Sound.LoadBGM( "sound/bgm/credits.mid", Sound.BGM_CREDITS, 128 )
	Sound.LoadBGM( "sound/bgm/level_01.mid", Sound.BGM_TITLE, 128 )
	
End Sub

sub Engine.LoadImages()
	
	PrintLog( 0, 160, 1, "Loading Images..." )
	
	Gl2D.FontLoad( 16, 16, 32, "gfx/consolas.bmp", GL_LINEAR )
	Explosion.Init( "gfx/flare.bmp" )
	
	OrbImage = GL2D.LoadBmpToGLsprite( "gfx/redorb.bmp" )
	
	
End Sub

sub Engine.ShutDown()
	
	GL2D.DestroyImage(OrbImage)
	
	Explosion.Release()
	GL2D.ShutDown()
	Sound.ShutDown()

End Sub
		
''*****************************************************************************
''
''	Main Gameplay state
''
''*****************************************************************************	
function Engine.StatePlay() as integer

	dim as integer Done = FALSE
	dt = GL2D.GetDeltaTime( FPS, timer )
	
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		MaxGeoSpawnDelay -= 0.01 * dt
		if( MaxGeoSpawnDelay < 1.0 )  then MaxGeoSpawnDelay = 1.0     '' Limit difficulty
		 
		''  Spawn enemies 
		SpawnEnemies()
		SpawnPowerUps()
		SpawnBonuses()
		
		''' Update input
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		'' Real-time volume control 
		if( Keys.Held( FB.SC_F1 ) ) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() - 1 )
		if( Keys.Held( FB.SC_F2 ) ) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() + 1 )
		
		'' Pause when we press enter
		if( Keys.Down( FB.SC_ENTER ) or Joy.KeyDown( JOY_OK ) ) then 
			State = STATE_PAUSE
			Done = FALSE
			Sound.PlaySFX( Sound.SFX_PAUSE )
		EndIf
		
		'' Yes or No when we press escape 
		if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			State = STATE_YES_OR_NO
			Done = FALSE
		EndIf
		
		'' Check lives
		dim as integer OldLife = Fighter.GetLives
		
		'' Update everything
		Asteroids.UpdateEntities( Fighter, dt )
		Asteroids.HandleCollisions( Fighter )
		
		PowerUps.UpdateEntities( Fighter, dt )
		PowerUps.HandleCollisions( Fighter )
		
		Bonuses.UpdateEntities( Fighter, dt )
		Bonuses.HandleCollisions( Fighter )
	
		Fighter.Update( Rat, Keys, Joy, dt, SecondsElapsed )
		if( Fighter.GetLives <= 0 ) then
			State = STATE_GAME_OVER
		else
			if (OldLife > Fighter.GetLives ) then
				PowSpawnDelay = 0      '' Spawn powerup when killed
			EndIf
		EndIf
		
		'' Reset difficulty
		if (OldLife > Fighter.GetLives ) then
			MaxGeoSpawnDelay = ENEMY_SPAWN_DELAY	
		EndIf
		
		
		Explosion.Update( dt )
		
		''  Update mouse for mouse-control
		MouseTarget.Reset()
		MouseTarget.Rotate( SecondsElapsed*2000 )
		MouseTarget.Translate( Rat.GetX, Rat.GetY )
		MouseTarget.Transform()
		
		'' Draw everything	
		Draw()
		
		'' FPS limit if capped
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
		
		
	Loop until ( State <> STATE_PLAY )
	
	
	return Done
	
End function


''*****************************************************************************
''
''	Paused state
''
''*****************************************************************************	

function Engine.StatePause() as integer
	
	dim as integer Done = FALSE
	
	Sound.PauseBGM( Sound.BGM_LEVEL_02 )
	
	dt = GL2D.GetDeltaTime( FPS, timer )
	
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		if( Keys.Held( FB.SC_F1 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() - 1 )
		if( Keys.Held( FB.SC_F2 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() + 1 )
		
		if( Keys.Down( FB.SC_ENTER ) or Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_OK )  or Joy.KeyDown( JOY_CANCEL ) ) then 
			State = STATE_PLAY
			Sound.PlaySFX( Sound.SFX_PAUSE )
		EndIf
		
		Draw()
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
				
	Loop until ( State <> STATE_PAUSE )

	Sound.UnPauseBGM( Sound.BGM_LEVEL_02 )
	
	return Done
	
End function

''*****************************************************************************
''
''	State called before Playing
''
''*****************************************************************************	

function Engine.StateStart() as integer
	
	dim as integer Done = FALSE
	
	''  Reset BG
	for i as integer = 0 to MAX_RINGS
		T(i).x = 320
		T(i).y = 240
		T(i).r = Lerp( 0, MAX_RADIUS , i / (MAX_RINGS + 1) )
	Next

	dt = GL2D.GetDeltaTime( FPS, timer )
	dim as single StartSec = SecondsElapsed
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		MouseTarget.Reset()
		MouseTarget.Rotate( SecondsElapsed*2000 )
		MouseTarget.Translate( Rat.GetX, Rat.GetY )
		MouseTarget.Transform()
		
		Draw()
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
				
	Loop until (SecondsElapsed - StartSec) > 3   '' Play after 3 seconds has elapsed
	
	''  Reset stuff
	Fighter.Initialize()
	Fighter.ReSpawn()
	Fighter.SetControlMode = ControlMode
	Fighter.SetAutoFire = AutoFire
	
	MaxGeoSpawnDelay = ENEMY_SPAWN_DELAY
	
	Sound.PlayBGM( Sound.BGM_LEVEL_02 )

	State = STATE_PLAY

	return Done
	
End function

''*****************************************************************************
''
''	Game over state
''
''*****************************************************************************	

function Engine.StateGameOver() as integer
	
	dim as integer Done = FALSE
	
	dt = GL2D.GetDeltaTime( FPS, timer )
	dim as single StartSec = SecondsElapsed
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		if( Keys.Held( FB.SC_F1 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() - 1 )
		if( Keys.Held( FB.SC_F2 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() + 1 )
		
		if( Keys.Down( FB.SC_ENTER ) or Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_OK ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			Done = TRUE
			Sound.PlaySFX( Sound.SFX_PAUSE )
		EndIf
		
		Asteroids.UpdateEntities( Fighter, dt )
		Asteroids.HandleCollisions( Fighter )
		
		PowerUps.UpdateEntities( Fighter, dt )
		PowerUps.HandleCollisions( Fighter )
		
		Fighter.Update( Rat, Keys, Joy, dt, SecondsElapsed )
		
		Explosion.Update( dt )
		
		Draw()
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
				
	Loop until Done or ((SecondsElapsed - StartSec) > 10)
	
	Done = FALSE
	
	State = STATE_TITLE

	return Done
	
End function

''*****************************************************************************
''
''	Options screen state
''
''*****************************************************************************	

function Engine.StateOptions() as integer
	
	dim as integer Rows(0 to ...) =>_
									{ 40  ,_  '' 0
									  100 ,_  '' 1
									  180 ,_  '' 2
									  220 ,_  '' 3
									  260 ,_  '' 4
									  300 ,_  '' 5
									  340 ,_  '' 7
									  380 ,_  '' 7
									  425 }   '' 8   
	
	dim as String Menu(0 to ...) =>_
									{ "BGM Volume:"             ,_  '' 0
									  "SFX Volume:"             ,_  '' 1
									  "Control Mode :"          ,_  '' 2
									  "Screen Mode  :"          ,_  '' 3
									  "Vsynch       :"          ,_  '' 4
									  "Cap FPS to 60:"          ,_  '' 5
									  "Show FPS     :"          ,_  '' 6
									  "AutoFire     :"          ,_  '' 7
									  "    Return to Title     " }  '' 8   
	
	dim as String Help(0 to ...) =>_
									{ "Change Volume of Background Music"  ,_  '' 0
									  "Change Volume of Sound Effects"  ,_  '' 1
									  "Player Control Mode" ,_  '' 2
									  "Full Screen or Windowed." ,_  '' 3
									  "Prevents GFX 'shearing'" ,_  '' 4
									  "Limit draw frames to 60 per second" ,_  '' 5
									  "Show number of frames per second" ,_  '' 6
									  "ON = Stingray Autofires" ,_  '' 7
									  "Save Config and go back to Title Screen" }	              '' 8   
	
	dim as integer MAX_CHOICE = ubound(Menu) + 1
	
	dim as integer Done = FALSE
	dim as integer Active = 0
	
	'' Pseudo elastique physics for the menu
	const as single Force = 10
	const as single Mass = 400
	const as single Friction = 0.821
	const as single Pull = Force/Mass
	const as single DistMag = 15
	const as single PullForce = Pull*DistMag
	    	
	dim as Vector2D MenuPixel = Vector2D( 0, 0 )
	dim as Vector2D MenuPixelSpring = Vector2D( 0, 0 )
	dim as Vector2D MenuPixelDir = Vector2D( 0, 0 )
	dim as single UpdateCoolDown = FIXED_TIME_STEP							
	
	dim as integer Save = FALSE
	
	Sound.PlayBGM( Sound.BGM_LEVEL_03 )

	dt = GL2D.GetDeltaTime( FPS, timer )	
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
	
		UpdateCoolDown -= dt
	
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		'' Update spring	
		if( UpdateCoolDown <= dt ) then
			MenuPixel = Vector2d( 0, Rows(Active) )
			dim as Vector2D D = (MenuPixelSpring - MenuPixel)
			MenuPixelDir -= D * PullForce
		    MenuPixelDir *= Friction
		    MenuPixelSpring += MenuPixelDir
		    UpdateCoolDown = FIXED_TIME_STEP
		endif
	    
		if( Keys.Held( FB.SC_LEFT ) or Joy.Left() ) then 
			select case Active
				case 0:
					Sound.SetMasterVolumeBGM( Sound.GetMasterVolumeBGM() - 1 )
				case 1:
					Sound.SetMasterVolumeSFX( Sound.GetMasterVolumeSFX() - 1 )
				case 2:
				case 3:
			End Select
		EndIf
		
		if( Keys.Held( FB.SC_RIGHT ) or Joy.Right() ) then 
			select case Active
				case 0:
					Sound.SetMasterVolumeBGM( Sound.GetMasterVolumeBGM() + 1 )
				case 1:
					Sound.SetMasterVolumeSFX( Sound.GetMasterVolumeSFX() + 1 )
				case 2:
				case 3:
			End Select
		EndIf
		
		if( Keys.Down( FB.SC_LEFT ) or Keys.Down( FB.SC_RIGHT ) or Joy.RightDown() or Joy.LeftDown() ) then 
			select case Active
				case 0:
				case 1:
				case 2:    	'' Control mode
					ControlMode = (ControlMode + 1) and 1 
					Sound.PlaySFX( Sound.SFX_CLICK )
				case 3:		'' Full Screen
					FullScreen = not FullScreen
					Sound.PlaySFX( Sound.SFX_CLICK )
					ResetScreen()
				case 4:		'' Vsynch
					Vsynch = not Vsynch
					Sound.PlaySFX( Sound.SFX_CLICK )
				case 5:		'' Cap FPS
					CapFPS = Not CapFPS
					Sound.PlaySFX( Sound.SFX_CLICK )
				case 6:		'' Show FPS
					ShowFPS = Not ShowFPS
					Sound.PlaySFX( Sound.SFX_CLICK )
				case 7:		'' AutoFire
					AutoFire = Not AutoFire
					Sound.PlaySFX( Sound.SFX_CLICK )
				case 8:		'' Exit
			End Select
		EndIf
		
		if( Keys.Down( FB.SC_ENTER ) or Joy.KeyDown( JOY_OK ) ) then 
			if( Active = 8 ) then
				Done = TRUE
				Save = TRUE
				Sound.PlaySFX( Sound.SFX_PAUSE )
			endif
		EndIf
		
		if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			Sound.PlaySFX( Sound.SFX_CLICK )
			Save = FALSE
			Done = TRUE
		EndIf
		
		if( Keys.Down( FB.SC_DOWN ) or Joy.DownDown() ) then
			Active = (Active + 1) mod MAX_CHOICE
			Sound.PlaySFX( Sound.SFX_CLICK )
		EndIf
		
		if( Keys.Down( FB.SC_UP ) or Joy.UpDown() ) then 
			Active = (Active + (MAX_CHOICE - 1)) mod MAX_CHOICE
			Sound.PlaySFX( Sound.SFX_CLICK )
		EndIf
		
		if( (int( SecondsElapsed * 2000 ) and 31) = 0 ) then
			Explosion.Spawn( Vector2D( rnd * SCREEN_WIDTH, rnd * SCREEN_HEIGHT ), 32 )
		endif
		
		Explosion.Update( dt )
		
		DrawOptions( Active, MenuPixelSpring, Rows(), Menu(), Help() )
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
				
	Loop until (Done)

	if( Vsynch ) then
		GL2D.VsyncOn()
	EndIf
	
	Sound.StopBGM( Sound.BGM_LEVEL_03 )

	Fighter.SetControlMode = ControlMode
	
	State = STATE_TITLE
	
	Done = FALSE

	Explosion.KillAll()
	
	MasterVolumeBGM = Sound.GetMasterVolumeBGM()
	MasterVolumeSFX = Sound.GetMasterVolumeSFX()
	
	if( Save ) then SaveConfig( "ConvexWars.cfg" )
	
	return Done
	
End function

''*****************************************************************************
''
''	Title state
''
''*****************************************************************************	

function Engine.StateTitle() as integer
	
	const as integer MAX_CHOICE = 5
	dim as integer Done = FALSE
	dim as integer Active = 0
	
	'' Pseudo elastique physics for the menu
	const as single Force = 10
	const as single Mass = 400
	const as single Friction = 0.821
	const as single Pull = Force/Mass
	const as single DistMag = 15
	const as single PullForce = Pull*DistMag
	    	
	dim as Vector2D MenuPixel = Vector2D( 0, 0 )
	dim as Vector2D MenuPixelSpring = Vector2D( 0, 0 )
	dim as Vector2D MenuPixelDir = Vector2D( 0, 0 )
	dim as single UpdateCoolDown = FIXED_TIME_STEP							
	
	
	Sound.StopBGM( Sound.BGM_LEVEL_01 )
	Sound.StopBGM( Sound.BGM_LEVEL_02 )
			
	Sound.PlayBGM( Sound.BGM_LEVEL_01 )

	Sound.SetMasterVolumeBGM( MasterVolumeBGM )
	Sound.SetMasterVolumeSFX( MasterVolumeSFX )
 	
	dt = GL2D.GetDeltaTime( FPS, timer )
	
	if( ControlMode = Ship.MODE_EXPERIMENTAL ) then Rat.SetVisible( FALSE )
		
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
	
		UpdateCoolDown -= dt
	
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		'' Update spring	
		if( UpdateCoolDown <= dt ) then
			MenuPixel = Vector2d( 100, 240 + Active * 40 )
			dim as Vector2D D = (MenuPixelSpring - MenuPixel)
			MenuPixelDir -= D * PullForce
		    MenuPixelDir *= Friction
		    MenuPixelSpring += MenuPixelDir
		    UpdateCoolDown = FIXED_TIME_STEP
		endif
	    
		if( Keys.Held( FB.SC_F1 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() - 1 )
		if( Keys.Held( FB.SC_F2 )) then Sound.SetMasterVolume( Sound.GetMasterVolumeBGM() + 1 )
		
		if( Keys.Down( FB.SC_ENTER ) or Joy.KeyDown( JOY_OK ) ) then 
			Done = TRUE
			Sound.PlaySFX( Sound.SFX_PAUSE )
		EndIf
		
		if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			Active = 4
			Sound.PlaySFX( Sound.SFX_CLICK )
			Done = TRUE
		EndIf
		
		if( Keys.Down( FB.SC_DOWN ) or Joy.DownDown() ) then
			Active = (Active + 1) mod MAX_CHOICE
			Sound.PlaySFX( Sound.SFX_CLICK )
		EndIf
		
		if( Keys.Down( FB.SC_UP ) or Joy.UpDown() ) then 
			Active = (Active + (MAX_CHOICE - 1)) mod MAX_CHOICE
			Sound.PlaySFX( Sound.SFX_CLICK )
		EndIf
		
		if( (int( SecondsElapsed * 2000 ) and 31) = 0 ) then
			Explosion.Spawn( Vector2D( rnd * SCREEN_WIDTH, rnd * SCREEN_HEIGHT ), 32 )
		endif
		
		Explosion.Update( dt )
		
		DrawTitle( Active, MenuPixelSpring )
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
				
	Loop until (Done)


	select case Active
		case 0:     '' Start
			State = STATE_START
			Done = FALSE
		case 1:     '' Options
			State = STATE_OPTIONS
			Done = FALSE
		case 2:     '' Credits
			State = STATE_CREDITS
			Done = FALSE
		case 3:     '' How to
			State = STATE_EXTRAS
			Done = FALSE
		case 4:     '' Exit
			State = STATE_EXIT
			Done = TRUE
	End Select
	
	Sound.StopBGM( Sound.BGM_LEVEL_01 )

	Fighter.Initialize()
	Asteroids.KillAllEntities()
	PowerUps.KillAllEntities()
	Bonuses.KillAllEntities()
	Explosion.KillAll()
	
	return Done
	
End function

function Engine.StateCredits() as integer
	
	'' I didn't know i can use this.
	dim as String Text(0 to ...) =>_
									{ "",_
									  "$Code" ,_
									  "Richard Eric M. Lope"    ,_
									  "[Relminator]"    ,_
									  "http://Rel.Phatcode.Net",_
									  "",_
									  "",_
									  "$Graphics",_
									  "Richard Eric M. Lope",_
									  "Anya Therese B. Lope",_
									  "",_
									  "",_
									  "$Music and SFX",_
									  "VgMusic.com",_
									  "Adigun A. Polack",_
									  "",_
									  "",_
									  "$Design",_
									  "Richard Eric M. Lope",_
									  "Anya Therese B. Lope",_
									  "",_
									  "",_
									  "$Tools",_
									  "Easy GL2D",_
									  "FreeBASIC",_
									  "OpenGL",_
									  "Fmod",_
									  "FBedit",_
									  "",_
									  "",_
									  "$Greetz",_
									  "God",_
									  "Rose Lope",_
									  "Peter Lope",_
									  "Lily Lope",_
									  "Marie Cristina Abejuela",_
									  "Cristina Marie Senosa",_
									  "Rich Abejuela",_
									  "Stanley Senosa",_
									  "Dr Davenstien",_
									  "V1ctor",_
									  "Plasma",_
									  "Jofers",_
									  "Piptol",_
									  "Jocke",_
									  "Lachie",_
									  "L_O_J",_
									  "Hellfire",_
									  "Shockwave",_
									  "X-Out",_
									  "0xbadc0de",_
									  "Ferofax",_
									  "Motorherp",_
									  "Vatoloco",_
									  "Jurassic Playeer",_
									  "Another World",_
									  "Zeromus",_
									  "Cearn",_
									  "Discostew",_
									  "Headkaze",_
									  "Flash",_
									  "Sverx",_
									  "Marc Russell",_
									  "Mj and Caleb Clark",_
									  "Blitz",_
									  "Nekrophidius",_
									  "Wildcard",_
									  "Dav",_
									  "Piptol",_
									  "Pritch",_
									  "Ryan",_
									  "",_
									  "",_
									  "$All the guys at:",_
									  "freebasic.net",_
									  "devkitpro.org",_
									  "gbatemp.net",_
									  "shmup-dev.com",_
									  "shmups.com",_
									  "symbianize.com",_
									  "gbadev.org",_
									  "And all those I forgot." }   
	
	dim as integer Done = FALSE
	dim as single Scroller = SCREEN_HEIGHT
	
	Sound.PlayBGM( Sound.BGM_CREDITS )

	dt = GL2D.GetDeltaTime( FPS, timer )
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		Scroller -= 50 * dt
		
		
		if( Scroller < ( -(ubound(Text)+5) * 20 ) )  then
			Done = TRUE
		EndIf
		
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		if( Keys.Down( FB.SC_ENTER ) or Joy.KeyDown( JOY_OK ) ) then 
			Done = TRUE
		EndIf
		
		if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			Done = TRUE
		EndIf
		
		dim as Vector2D Position
		Position.x = (SCREEN_WIDTH/2 ) + ( sin( SecondsElapsed * 2) + cos( SecondsElapsed    ) ) * (SCREEN_WIDTH/2 ) 
		Position.y = (SCREEN_HEIGHT/2) + ( cos( SecondsElapsed    ) + cos( SecondsElapsed * 3) ) * (SCREEN_HEIGHT/2) 
		Explosion.Spawn(Position, 18 )	
		Explosion.Update( dt )
		
		GL2D.ClearScreen()
		
		GL2D.Begin2D()
		
			Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
			Explosion.DrawAll()
		
			glColor4ub(255,255,255,255)
			
			Gl2D.SetBlendMode( GL2D.BLEND_ALPHA )
			for i as integer = 0 to ubound(Text)
				dim as string s = Text(i)
				if( left(s,1) = "$" ) then 
					glColor4ub(0,0,255,255)
					CenterText( Scroller + i * 20-20, 2, right( s, Len(s) - 1 ) )
				else
					glColor4ub(255,255,255,255)
					CenterText( Scroller + i * 20, 1, s )
				EndIf
		    	
		    	
			next i	
			
			GL2D.SetBlendMode( GL2D.BLEND_ALPHA )
			GL2D.BoxFilledGradient( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT,_
									GL2D_RGBA( 12,0,0,255 ),_
									GL2D_RGBA( 0,0,0,0   ),_
									GL2D_RGBA( 0,0,0,0   ),_
									GL2D_RGBA( 12,0,0,255 ) )
			
			
		GL2D.End2D()
	
		sleep 1,1
		flip
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
					
	Loop until ( Done )
	
	Sound.StopBGM( Sound.BGM_CREDITS )

	State = STATE_TITLE
	Done = FALSE
	
	return Done
	
End function

function Engine.StateExtras() as integer
	
	'' I didn't know i can do something like this.
	dim as String Text(0 to ...) =>_
									{ "",_
									  "$Ship Controls:" ,_
									  "@Keyboard/Joystick"    ,_
									  "   UP/DOWN    = Accelerate/Decelerate",_
									  "   RIGHT/LEFT = Rotate",_
									  "   SPACE/Z/X  = Fire",_
									  "   J1/J2      = Fire",_
									  "@Mouse+(Keyboard/Joystick)",_
									  "   UP/DOWN/W/S    = Accelerate/Decelerate",_
									  "   RIGHT/LEFT/A/D = Strafe-Rotate",_
									  "   SPACE/Z/X      = Fire",_
									  "   J1/J2          = Fire",_
									  "   MOUSE BTNS     = Fire",_
									  "",_
									  "",_
									  "$Game Controls:",_
									  "   ESCAPE    = Quit/Cancel",_
									  "   ENTER     = Choose",_
									  "   F1        = Decrease Volume",_
									  "   F2        = Increase Volume",_
									  "   SPACE     = Pause",_
									  "   JOY BTNS  = Either Ok or Cancel",_
									  "   UP/DOWN   = Scroll this screen *;)",_
									  "   JUP/JDOWN = Scroll this screen *;)",_
									  "",_
									  "",_
									  "$Game Entities:",_
									  "",_
									  "@Ship",_
									  "",_
									  "",_
									  "   Codenamed the 'Stingray'",_
									  "   Faster than a bullet.",_
									  "   More Powerful than a speeding locomotive.",_
									  "   Shoots everything that moves.",_
									  "",_
									  "@Enemies",_
									  "",_
									  "",_
									  "   Small enemies are just cannon fodder." ,_
									  "   Large ones however, spawn smaller enemies",_
									  "   when destroyed.",_
									  "",_
									  "@PowerUp",_
									  "",_
									  "",_
									  "   Powers up the stingray for more shots.",_
									  "   Maximum shot-level is 3 and succeeding",_
									  "   powerups just give you additional score.",_
									  "",_
									  "@Bonus",_
									  "",_
									  "",_
									  "   Get these to increase your score.",_
									  "",_
									  "",_
									  "$Tips and Tricks:",_
									  "   If it moves, shoot it, it it doesn't,",_
									  "   shoot it anyway.",_
									  "",_
									  "" }   
	
	dim as Ship StingRay
	dim as Polygon Badguys(3)
	dim as Polygon Pow
	dim as Polygon Button(1)
	
	const as integer MIN_SCROLL = 50
	dim as integer Done = FALSE
	dim as single Scroller = MIN_SCROLL
	
	Sound.PlayBGM( Sound.BGM_LEVEL_03 )

	StingRay.Initialize()
	
	Badguys(0).Create( 17, 10, 1.3 )
	Badguys(1).Create( 3, 20, 3 )
	Badguys(2).Create( 6, 30, 1.5 )
	Badguys(3).Create( 5, 40 )
	
	Pow.Create( 8, 20 )
	
	Button(0).Create( 3, 20, 1, 3 )
	Button(1).Create( 3, 20, 1, 3 )
			
	dt = GL2D.GetDeltaTime( FPS, timer )
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
		
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		if( Keys.Held( FB.SC_UP ) or Joy.Up() ) then 
			if( Scroller < MIN_SCROLL ) then Scroller += 250 * dt
		EndIf
		
		if( Keys.Held( FB.SC_DOWN ) or Joy.Down() ) then 
			if( Scroller > -720 ) then Scroller -= 250 * dt
		EndIf
		
		if( Keys.Down( FB.SC_ENTER ) or Joy.KeyDown( JOY_OK ) ) then 
			Done = TRUE
		EndIf
		
		if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
			Done = TRUE
		EndIf
		
		GL2D.ClearScreen()
		
		GL2D.Begin2D()
		
			glColor4ub(255,255,255,255)
			
			Gl2D.SetBlendMode( GL2D.BLEND_ALPHA )
			for i as integer = 0 to ubound(Text)
				dim as string s = Text(i)
				if( left(s,1) = "$" ) then 
					glColor4ub(255,0,255,255)
					GL2D.PrintScale( 0, Scroller + i * 20-20, 1.5, right( s, Len(s) - 1 ) )
				elseif( left(s,1) = "@" ) then 
					glColor4ub(0,255,255,255)
					GL2D.PrintScale( 30, Scroller + i * 20-5, 1.2, right( s, Len(s) - 1 ) )
				else
					glColor4ub(255,255,0,255)
					GL2D.PrintScale( 30, Scroller + i * 20, 0.8, s )
				EndIf
		    	
		    	
			next i	
			
			
			Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
			
			'' Stingray
			Stingray.Reset()
			Stingray.Rotate( 270 )
			Stingray.Translate( 150, 570 + Scroller )
			StingRay.TransForm()
			StingRay.Draw( abs(sin(SecondsElapsed * 5)) * 4 )
			
			'' Polygons
			for i as integer = 0 to ubound( Badguys )
				Badguys(i).Reset()
				Badguys(i).Rotate( (i * 20) + SecondsElapsed * 50 )
				Badguys(i).Translate( 200 + i * 100, 740 + Scroller )
				Badguys(i).TransForm()			
			Next
			dim as single Intensity = abs(sin(SecondsElapsed * 4)) * 4
			for i as integer = 0 to 4
				Badguys(0).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 255 ) )
				Badguys(1).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 0, 255, 64, 255 ) )
				Badguys(2).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 255, 0, 255, 255 ) )
				Badguys(3).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 0, 255, 255, 255 ) )
			next i
			
			'' PowerUp
			Pow.Reset()
			Pow.Rotate( SecondsElapsed * 150 )
			Pow.Translate( 220 , 875 + Scroller )
			Pow.TransForm()			
			
			Intensity = 30 + abs(sin(SecondsElapsed * 2)) * 5
			for i as integer = 0 to 2
				Pow.Draw( 8 + (i * Intensity), GL2D_RGBA( 255, 255, 0, 255 ) )
			next i
	
			'' Bonus
			for j as integer = 0 to 2
				Pow.Reset()
				Pow.Rotate( SecondsElapsed * 150 )
				Pow.Translate( 170 + (j * 100 ) , 1020 + Scroller )
				Pow.TransForm()				
				Intensity = 10 + abs(sin(SecondsElapsed * (j+1))) * 10
				for i as integer = 0 to 2
					select case j
						case 0:
							Pow.Draw( 8 + (i * Intensity), GL2D_RGBA( 255,   0,   0, 255 ) )
						case 1:
							Pow.Draw( 8 + (i * Intensity), GL2D_RGBA(   0, 255,   0, 255 ) )
						case 2:
							Pow.Draw( 8 + (i * Intensity), GL2D_RGBA(   0,   0, 255, 255 ) )
					End Select
				next i
			next j
			
			'' Buttons
			Button(0).Reset()
			Button(0).Rotate( 270 )
			Button(0).Translate( 580, 25 )
			Button(0).TransForm()
			
			Button(1).Reset()
			Button(1).Rotate( 90 )
			Button(1).Translate( 580, SCREEN_HEIGHT-25 )
			Button(1).TransForm()
			
			Intensity = abs(sin(SecondsElapsed * 12)) * 4
			for i as integer = 0 to 4
				Button(0).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 255 ) )
				Button(1).Draw( 10 - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 255 ) )
			next i
			
			if( Keys.Held( FB.SC_UP ) or Joy.Up() ) then
				for i as integer = 0 to 10
					Button(0).Draw( 40 - ( i * Intensity*2 ), GL2D_RGBA( 255, 64, 64, 255 ) )
				next i
			EndIf
			
			if( Keys.Held( FB.SC_DOWN ) or Joy.Down() ) then
				for i as integer = 0 to 10
					Button(1).Draw( 40 - ( i * Intensity*2 ), GL2D_RGBA( 255, 64, 64, 255 ) )
				next i
			EndIf
			
			Gl2D.SetBlendMode( GL2D.BLEND_ALPHA )
			CenterText( 0, 2, "How to Play" )
			
		GL2D.End2D()
	
		sleep 1,1
		flip
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
					
	Loop until ( Done )
	
	Sound.StopBGM( Sound.BGM_LEVEL_03 )

	State = STATE_TITLE
	Done = FALSE
	
	return Done
	
End function


function Engine.StateYesOrNo() as integer
	
	dim as integer Done = FALSE
	dim as single t = 0
	dim as integer flow = 0
	dim as integer Cols(1) => {130, 350}
	dim as integer Active = 1
	Sound.PlaySFX( Sound.SFX_PAUSE )
	dt = GL2D.GetDeltaTime( FPS, timer )
	do
		
		dt = GL2D.GetDeltaTime( FPS, timer )
		Frame += 1
		SecondsElapsed += 1 * dt
	
		Rat.ScanButtons()
		Joy.ScanButtons()
		
		if( t >= 1 ) then
			if( Keys.Down( FB.SC_ENTER )  or Joy.KeyDown( JOY_OK ) ) then 
				flow = 1
				Sound.PlaySFX( Sound.SFX_CLICK )
			EndIf
			
			if( Keys.Down( FB.SC_ESCAPE ) or Joy.KeyDown( JOY_CANCEL ) ) then 
				flow = 1
				Active = 1
			EndIf
			
			if( Keys.Down( FB.SC_RIGHT ) or Joy.RightDown() ) then
				Active = (Active + 1) and 1
				Sound.PlaySFX( Sound.SFX_CLICK )
			EndIf
		
			if( Keys.Down( FB.SC_LEFT ) or Joy.LeftDown() ) then
				Active = (Active - 1) and 1
				Sound.PlaySFX( Sound.SFX_CLICK )
			EndIf
		
		endif
		
		if( flow = 0 ) then
			t += dt*7
			if( t > 1 ) then t = 1
		else
			t -= dt*7
			if( t < 0 ) then 
				t = 0
				State = STATE_PLAY
			EndIf
		endif
		
		Draw( FALSE )
			
		GL2D.Begin2D()
		
			glColor4ub(255,255,255,255)
			
			Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
		    dim as integer Px = SCREEN_WIDTH - 320 * t
			GL2D.SpriteStretch( Px\2-48, 150,  64 + 350 * t,  200, OrbImage )
			
			Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
		    glColor4ub(255,255,255,255)
			CenterText( 190, t,"Return to Title?" )
			CenterText( 270, t*2 ,"YAR    NAY" )
			glColor4ub(0,0,255,255)
			Gl2D.PrintScale( Cols(Active), 270, t * 2, "<   >" )
				
		GL2D.End2D()
	
		sleep 1,1
		flip
		
		if( CapFPS ) then dim as integer FPS2 = GL2D.LimitFPS(60)
					
	Loop until ( State <> STATE_YES_OR_NO )
	
	if( Active = 0 ) then  '' yes
		State = STATE_TITLE
	EndIf
	
	return Done
	
End function


''*****************************************************************************
''
''
''
''*****************************************************************************
sub Engine.Draw( byval FlipIt as integer = TRUE )
	
	dim as integer Blinker = (int( SecondsElapsed * 2 ) and 1)
	
	GL2D.ClearScreen()
		
	GL2D.Begin2D()
	
		glColor4ub(255,255,255,255)
		
		Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
	    
	    DrawWeb()
	    
	    Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
	    
	    Asteroids.DrawEntities()
	    
	    PowerUps.DrawEntities()
	    
	    Bonuses.DrawEntities()
	    
	    Fighter.Draw( abs(sin(timer * 5)) * 4 )
	    
	    if( ControlMode = Ship.MODE_EXPERIMENTAL ) then
				dim as single intensity = abs(sin( SecondsElapsed * 6 )) * 7
				MouseTarget.Draw( 15 + intensity, GL2D_RGBA(0, 255, 255, 255) )
				GL2D.LineGlow( Rat.GetX, Rat.GetY - 15, Rat.GetX, Rat.GetY + 15, 12, GL2D_RGBA( 255, 0, 0, 255 ) )
				GL2D.LineGlow( Rat.GetX - 15, Rat.GetY, Rat.GetX + 15, Rat.GetY, 12, GL2D_RGBA( 255, 0, 0, 255 ) )
		endif
			
	    Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
	    Explosion.DrawAll()
	    
	    '' Difficulty level
	    dim as single l = ( (ENEMY_SPAWN_DELAY-1) - (MaxGeoSpawnDelay-1)) / (ENEMY_SPAWN_DELAY-1)	
		if( l > 0.3 ) and ( l <= 0.6 ) then
			GL2D.LineGlow( 180, 472, 180 + Lerp(0,SCREEN_WIDTH/2, l), 472, 16, GL2D_RGBA( 255, 255, 255, 180 ), OrbImage  )
		elseif( l > 0.6 ) and ( l <= 0.9 ) then
			GL2D.LineGlow( 180, 472, 180 + Lerp(0,SCREEN_WIDTH/2, l), 472, 16, GL2D_RGBA( 255, 255, 255, 230 ), OrbImage  )
		elseif( l > 0.9 ) then
			GL2D.LineGlow( 180, 472, 180 + Lerp(0,SCREEN_WIDTH/2, l), 472, 16, GL2D_RGBA( 255, 255, 255, 255 ), OrbImage  )
		else
			GL2D.LineGlow( 180, 472, 180 + Lerp(0,SCREEN_WIDTH/2, l), 472, 16, GL2D_RGBA( 255, 255, 255, 128 ), OrbImage  )
		endif
		
		Gl2D.SetBlendMode( GL2D.BLEND_GLOW )
		if( l > 0.9 ) then
			dim as single intensity = abs(sin( SecondsElapsed * 20 ))
			GL2D.LineGlow( 180, 472, 180 + Lerp(0,SCREEN_WIDTH/2, l), 472, 16, GL2D_RGBA( 255 - 255 * Intensity, 255 - 255 * Intensity, 255 * Intensity, 255 * Intensity ), OrbImage  )
		endif
		
		Gl2D.SetBlendMode( GL2D.BLEND_TRANS )
		dim as integer lives = Fighter.GetLives - 1
		if( lives < 0 ) then lives = 0
	    Gl2D.PrintScale( 0, 0, 1, "Ships = " & lives )
		Gl2D.PrintScale( 404, 0, 1, "Score =" )
		PrintScore( 640 - (16 * 7), 0, 1, Fighter.GetScore, 7, "0" )
		if( ShowFPS) then Gl2D.PrintScale( 514, 464, 1, "FPS:" & str(int(FPS)) )
		Gl2D.PrintScale( 0, 464, 1, "Difficulty:[0]   [1]   [2]  [3]" )
			
		select case State
			
			case STATE_PLAY:
			case STATE_PAUSE:
				if( Blinker ) then
					glColor4ub(255,255,0,255)
					CenterText( 200, 3 ,"PAUSED" )
					glColor4ub(255,255,255,255)
				endif
			case STATE_START:
				if( Blinker ) then
					glColor4ub(0,255,255,255)
					CenterText( 200, 2.5 ,"READY" )
					glColor4ub(255,255,255,255)
				endif
			case STATE_GAME_OVER:
				if( Blinker ) then
					glColor4ub(0,255,255,255)
					CenterText( 200, 2.5 ,"G A M E  O V E R" )
					glColor4ub(255,255,255,255)
				endif
			case STATE_OPTIONS:
			case STATE_EXTRAS:
			case STATE_CREDITS:
			case STATE_TITLE:
			case STATE_EXIT:
				
		End Select
		
	GL2D.End2D()
	
	if( FlipIt ) then
	
		sleep 1,1
		flip
		
	endif
	
End Sub

sub Engine.DrawTitle( byval active as integer, byref v as Vector2D )
	
	dim as integer Blinker = (int( SecondsElapsed ) and 1)
	const as integer MAX_CHOICE = 5
	
	dim as String Menu(MAX_CHOICE - 1) =>_
									{ "Start Game" ,_  '' 0
									  "Options"    ,_  '' 1
									  "Credits"    ,_  '' 2
									  "How to Play",_  '' 3
									  "Exit Game" }	   '' 4   
	
	dim as String Help(MAX_CHOICE - 1) =>_
									{ "Starts a New Game." 				,_  '' 0
									  "Set Game Options."    			,_  '' 1
									  "See Credits and Greetings."    	,_  '' 2
									  "Instructions on How to Play."	,_  '' 3
									  "Choose this if the game suxxors." }	   '' 4   
	
	GL2D.ClearScreen()
		
	GL2D.Begin2D()
	
		glColor4ub(0,255,0,255)
		
		Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
		Explosion.DrawAll()
	    
		dim as single col = abs( sin( SecondsElapsed * 4 ) ) * 255
	    glColor4ub(0,255,0,255)
		CenterText( 10, 3 ,"C O N V E X" )
		CenterText( 70, 3 ,"W A R S" )
		glColor4ub(255,255,255,col)
		CenterText( 10, 3 ,"C O N V E X" )
		CenterText( 70, 3 ,"W A R S" )
		
		
		Gl2D.SetBlendMode( GL2D.BLEND_TRANS )
		glColor4ub(255,0,255,255)
		CenterText( 140, 1 ,"CODE::DESIGN" )
		CenterText( 160, 1 ,"Richard Eric M. Lope" )
		CenterText( 180, 1 ,"Relminator" )
		CenterText( 200, 1 ,"HTTP://REL.PHATCODE.NET" )
		
		glColor4ub(255,255,255,255)
		for i as integer = 0 to MAX_CHOICE - 1
			CenterText( 240 + i * 40, 1.5 , Menu(i) )
		Next

		glColor4ub(0,0,255,255)
		dim as string a = "<[" & Space( Len(Menu(Active)) ) & "]>"
		CenterText( v.y, 1.5 , a )
		Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
		glColor4ub(255,255,255,col)
		CenterText( v.y, 1.5 , a )
		
		glColor4ub(255,0,0,255)
		CenterText( 240 + Active * 40, 1.5 , Menu(Active) )
		CenterText( 440, 1, Help(Active) )

		Gl2D.SetBlendMode( GL2D.BLEND_TRANS )
		glColor4ub(255,255,0,255)
		if( Blinker ) then
			CenterText( 460, 1, "Press [ENTER] to Select Choice." )
		else
			CenterText( 460, 1, "Press [ESCAPE] to Exit Game." )
		endif

	GL2D.End2D()
	
	sleep 1,1
	
	flip
	
End Sub

sub Engine.DrawOptions( byval active as integer, byref v as Vector2D, Rows() as integer, Menu() as string, Help() as string )
	
	dim as integer Blinker = (int( SecondsElapsed ) and 1)
	dim as integer MAX_CHOICE = ubound(Menu) + 1
	
	GL2D.ClearScreen()
		
	GL2D.Begin2D()
	
		glColor4ub(0,255,0,255)
		
		Gl2D.SetBlendMode( GL2D.BLEND_BLENDED )
		Explosion.DrawAll()
	    
		dim as single col = abs( sin( SecondsElapsed * 4 ) ) * 255
	    glColor4ub(255, 0,0,255)
		CenterText( 10, 2,"OPTIONS" )
		glColor4ub(255,255,255,col)
		CenterText( 10, 2 ,"OPTIONS" )
		
		
		GL2D.SetBlendMode( GL2D.BLEND_TRANS )
		
		glColor4ub(255,255,255,255)
		for i as integer = 0 to MAX_CHOICE - 1
			GL2D.PrintScale( 20, Rows(i), 1.5, Menu(i) )
		Next

		glColor4ub(0,0,255,255)
		dim as string a = "[" & Space( Len(Menu(Active)) - 1 ) & "]"
		GL2D.PrintScale( 5, v.y , 1.5, a )
		GL2D.SetBlendMode( GL2D.BLEND_BLENDED )
		glColor4ub(255,255,255,col)
		GL2D.PrintScale( 5, v.y, 1.5, a )
		
		glColor4ub(0,255,0,255)
		GL2D.PrintScale( 20, Rows(Active), 1.5, Menu(Active) )
		CenterText( 466, 1, Help(Active) )
		
		
		GL2D.SetBlendMode( GL2D.BLEND_GLOW )
		glColor4ub(255,255,255,255)
		
		dim as single Length = Lerp( 1, 550, Sound.GetMasterVolumeBGM/255 )
		GL2D.LineGlow( 40, Rows(0) + 35, 40 + Length, Rows(0) + 35, 32, GL2D_RGBA( 255, 255, 255, 255 ), OrbImage  )
		
		Length = Lerp( 1, 550, Sound.GetMasterVolumeSFX/255 )
		GL2D.LineGlow( 40, Rows(1) + 35, 40 + Length, Rows(1) + 35, 32, GL2D_RGBA( 255, 255, 255, 255 ), OrbImage  )
		
		'' Control Mode
		glColor4ub(255,255,0,255)
		if( ControlMode = Ship.MODE_NORMAL ) then
			GL2D.PrintScale( 365, Rows(2) + 4, 1, "KEYBOARD/JOYSTICK" )
		else
			GL2D.PrintScale( 365, Rows(2) + 4, 1, "KEYBOARD+MOUSE" )
		endif	
		
		'' Full Screen
		if( FullScreen ) then
			GL2D.PrintScale( 365, Rows(3) + 4, 1, "FULL SCREEN" )
		else
			GL2D.PrintScale( 365, Rows(3) + 4, 1, "WINDOWED" )
		endif
		
		'' Vsynch
		if( Vsynch ) then
			GL2D.PrintScale( 365, Rows(4) + 4, 1, "ON" )
		else
			GL2D.PrintScale( 365, Rows(4) + 4, 1, "OFF" )
		endif
		
		'' Cap FPS
		if( CapFPS ) then
			GL2D.PrintScale( 365, Rows(5) + 4, 1, "YES" )
		else
			GL2D.PrintScale( 365, Rows(5) + 4, 1, "NO" )
		endif
		
		'' Show FPS
		if( ShowFPS ) then
			GL2D.PrintScale( 365, Rows(6) + 4, 1, "YES" )
		else
			GL2D.PrintScale( 365, Rows(6) + 4, 1, "NO" )
		endif
		
		'' AutoFire
		if( AutoFire ) then
			GL2D.PrintScale( 365, Rows(7) + 4, 1, "ON" )
		else
			GL2D.PrintScale( 365, Rows(7) + 4, 1, "OFF" )
		endif
		
		GL2D.SetBlendMode( GL2D.BLEND_GLOW )
		glColor4ub(0,255,255,255)
			
		if( Blinker ) then
			if( Active < 8 ) then
				CenterText( Rows(Active) + 27, 1, "LEFT/RIGHT to Change Values" )
			else
				CenterText( Rows(Active) + 27, 1, "ENTER to Save and Return to Title" )
			endif
		else
			CenterText( Rows(Active) + 27, 1, "ESCAPE to Return to Title" )
		EndIf
	
	GL2D.End2D()
	
	sleep 1,1
	
	flip
	
End Sub


sub Engine.DrawWeb()

	for i as integer = 0 to MAX_RINGS
		for j as integer = 0 to 359 step 20
			dim as single x1 = T(i).x + cos(DEG2RAD(j)) * T(i).r
			dim as single y1 = T(i).y + sin(DEG2RAD(j)) * T(i).r
			dim as single x2 = T(i).x + cos(DEG2RAD(j+20)) * T(i).r
			dim as single y2 = T(i).y + sin(DEG2RAD(j+20)) * T(i).r
			dim as integer col  = (T(i).r)/MAX_RADIUS * 100
			for w as integer = 0 to 2
				GL2D.LineGlow( x1, y1, x2, y2, (T(i).r/30) - w, GL2D_RGBA(0,col + w,0,col))
			next
			
			dim as integer k = (i + 1) mod (MAX_RINGS + 1)
			if( T(k).r > T(i).r) then
				x2 = T(k).x + cos(DEG2RAD(j)) * T(k).r
				y2 = T(k).y + sin(DEG2RAD(j)) * T(k).r
				for w as integer = 0 to 2
					GL2D.LineGlow( x1, y1, x2, y2, (T(k).r/30)  - w , GL2D_RGBA(0,col + w,0,col))
				next			
			endif
		Next
	Next

	for i as integer = 0 to MAX_RINGS
		T(i).r += 1000 * dt
		if( T(i).r > MAX_RADIUS ) then
			T(i).x = 320 + (cos( SecondsElapsed * 4 ) + sin( SecondsElapsed * 2 )) * 150
			T(i).y = 240 + (sin( SecondsElapsed * 3 ) + sin( SecondsElapsed * 4 )) * 150
			T(i).r = 0
		EndIf
	Next

				
End Sub
	

sub Engine.SpawnEnemies()
	
	'' Spawn GeoStigmas
	GeoSpawnDelay -= 1 * dt
	if( GeoSpawnDelay <= 0 ) then

		GeoSpawnDelay = MaxGeoSpawnDelay
	
		dim as integer NumVerts = 3 + rnd * 8
		dim as single Radius = 10 + rnd * 40
		dim as single ScaleX = 0.3f + rnd * 2.0f
		dim as single Speed = 20 + rnd * 80
		dim as integer Colour = GL2D_RGBA( 127 + Rnd * 128, 127 + Rnd * 128, 127 + Rnd * 128, 255 )
		
		dim as single Angle = rnd * ( 2.0f * PI )
		dim as integer Vr = 350
		dim as Vector2D Posi = Vector2D( SCREEN_WIDTH/2.0f + cos(Angle) * Vr,_
								         SCREEN_HEIGHT/2.0f + sin(Angle) * Vr )
		
		dim as Vector2D Target = Vector2D( 50 + rnd * 590, 50 + rnd * 430 )
		
		Asteroids.Spawn( Posi, Target,_
						 Speed, Colour,_
						 NumVerts, Radius, ScaleX, 1.0f )
	
	endif
	
End Sub

sub Engine.SpawnPowerUps()
	
	'' Spawn Powerups
	if( (PowerUps.GetNumEntities + Fighter.GetLevel) < 3 ) then
		PowSpawnDelay -= 1 * dt
		if( PowSpawnDelay <= 0 ) then
			PowSpawnDelay = MaxPowSpawnDelay	
			dim as Vector2D Posi = Vector2D( 20 + rnd * (SCREEN_WIDTH - 40), 20 + rnd * (SCREEN_HEIGHT - 40) )
			PowerUps.Spawn( Posi )
		endif
	endif
	
End Sub

sub Engine.SpawnBonuses()
	
	'' Spawn Powerups
	if( Bonuses.GetNumEntities = 0 ) then
		BonusSpawnDelay -= 1 * dt
		if( BonusSpawnDelay <= 0 ) then
			BonusSpawnDelay = MaxBonusSpawnDelay	
			dim as Vector2D Posi = Vector2D( 20 + rnd * (SCREEN_WIDTH - 40), 20 + rnd * (SCREEN_HEIGHT - 40) )
			Bonuses.Spawn( Posi )
		endif
	endif
	
End Sub

sub Engine.CenterText( byval y as integer, byval scale as single, byref text as string, byval charwid as integer = 16 )

	charwid *= scale

	dim as integer col = (SCREEN_WIDTH - (charwid * len( text )))/2

	Gl2D.PrintScale( col, y, scale, text )
	
End Sub
	
	
sub Engine.PrintLog( byval x as integer, byval y as integer, byval scale as single, byref text as string )
	
	GL2D.Begin2D()
	
		Gl2D.SetBlendMode( GL2D.BLEND_GLOW )		
		Gl2D.PrintScale( x, y, scale, text )
				
	GL2D.End2D()
	sleep 1,1
	flip
	
End Sub

sub Engine.PrintScore( byval x as integer, byval y as integer, byval scale as single, byval sc as integer, byval numchars as integer, byref filler as string )
	
	dim as string score = str(sc)
	dim as string text = string(numchars - len(score), filler) 
	
	text = text & score
	Gl2D.PrintScale( x, y, scale, text )
	
End Sub

sub Engine.ResetScreen()
	
	dim as integer flag = FB.GFX_WINDOWED
	if( FullScreen ) then flag = FB.GFX_FULLSCREEN
	
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT, flag or FB.GFX_NO_FRAME )   ''Set up GL screen
	
	GL2D.EnableAntialias( TRUE )

	LoadImages()
	
End Sub

sub Engine.SaveConfig( byref filename as string )
	
	dim as integer f = FreeFile

	if( open( filename for binary as #f ) = 0 ) then
		
		put #f,, Sound.GetMasterVolumeBGM
		put #f,, Sound.GetMasterVolumeSFX
		put #f,, ControlMode
		put #f,, FullScreen
		put #f,, Vsynch
		put #f,, CapFPS 
		put #f,, ShowFPS
		put #f,, AutoFire 
		close #f

	endif
	
	
End Sub

sub Engine.LoadConfig( byref filename as string )
	
	dim as integer f = FreeFile
	if( open( filename for binary as #f ) = 0 ) then
		
		get #f,, MasterVolumeBGM
		get #f,, MasterVolumeSFX
		get #f,, ControlMode
		get #f,, FullScreen
		get #f,, Vsynch
		get #f,, CapFPS
		get #f,, ShowFPS
		get #f,, AutoFire 
		
		close #f
	 
	endif

	Sound.SetMasterVolumeBGM( MasterVolumeBGM )
	Sound.SetMasterVolumeSFX( MasterVolumeSFX )
	
	
	
End Sub
	