''*****************************************************************************
''
''
''	Engine class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "FBGFX.bi"
#include once "FBGL2D7.bi"

#include once "Vector2D.bi"
#include once "Matrix2D.bi"
#include once "Polygon.bi"
#include once "Sound.bi"
#include once "Explosion.bi"

#include once "Globals.bi"
#include once "Ship.bi"
#include once "Asteroid.bi"
#include once "PowerUp.bi"
#include once "Bonus.bi"

''*****************************************************************************
''
''
''
''*****************************************************************************
#define MAX_RINGS 10

'' Crappy effect for a web-like BG
type Tunnel
	as single x
	as single y
	as single r
End Type

type Engine
	
public:
	'' We are going to use FSM for the engine itself
	enum 
		STATE_PLAY = 0,
		STATE_PAUSE,
		STATE_START,
		STATE_GAME_OVER,
		STATE_OPTIONS,
		STATE_EXTRAS,
		STATE_CREDITS,
		STATE_TITLE,
		STATE_YES_OR_NO,
		STATE_EXIT 
	end enum
	
	''Menu choices
	enum 
		CHOICE_START_GAME = 0,
		CHOICE_OPTIONS,
		CHOICE_EXTRAS,
		CHOICE_CREDITS,
		CHOICE_EXIT
	end enum

	
	declare constructor()
	declare destructor()
	
	declare property SetState( byval v as integer )
	
	declare sub Initialize()
	declare function Update() as integer
	declare sub InitEverything()
	declare sub LoadSounds()
	declare sub LoadImages()
	declare sub ShutDown()
	
	
	
private:
	declare function StatePlay() as integer     '' Engine State Functions
	declare function StatePause() as integer
	declare function StateStart() as integer
	declare function StateGameOver() as integer
	declare function StateOptions() as integer
	declare function StateTitle() as integer
	declare function StateCredits() as integer
	declare function StateExtras() as integer
	declare function StateYesOrNo() as integer
	
	declare sub Draw( byval FlipIt as integer = TRUE )
	declare sub DrawTitle( byval active as integer, byref v as Vector2D )
	declare sub DrawOptions( byval active as integer, byref v as Vector2D, Rows() as integer, Menu() as string, Help() as string )
	declare sub DrawWeb()
	
	declare sub SpawnEnemies()			'' Timer based entity spawners
	declare sub SpawnPowerUps()
	declare sub SpawnBonuses()
	
	'' Some nice stuff
	declare sub CenterText( byval y as integer, byval scale as single, byref text as string, byval charwid as integer = 16 )
	declare sub PrintLog( byval x as integer, byval y as integer, byval scale as single, byref text as string )
	declare sub PrintScore( byval x as integer, byval y as integer, byval scale as single, byval sc as integer, byval numchars as integer, byref filler as string )
	declare sub ResetScreen()
	
	declare sub SaveConfig( byref filename as string )
	declare sub LoadConfig( byref filename as string )
	
	
	as integer 	State
	as integer 	Level
	as integer 	Frame
	as single 	SecondsElapsed
	as single 	FPS
	as single 	dt
	
	as single 	GeoSpawnDelay  		'' Spawn counters
	as single 	MaxGeoSpawnDelay
	as single 	PowSpawnDelay
	as single 	MaxPowSpawnDelay
	as single 	BonusSpawnDelay
	as single 	MaxBonusSpawnDelay
	
	as integer 	ControlMode
	as integer  FullScreen
	as integer 	Vsynch
	as integer 	CapFPS
	as integer 	ShowFPS
	as integer 	AutoFire
	
	as integer  MasterVolumeBGM
	as integer  MasterVolumeSFX
	
	as Mouse 	Rat
	as Keyboard Keys
	as Joystick Joy
	
	as Ship 	Fighter
	
	as AsteroidFactory Asteroids
	as PowerUpFactory PowerUps
	as BonusFactory Bonuses
	
	as GL2D.Image ptr OrbImage
	as Tunnel T(MAX_RINGS)
	as Polygon MouseTarget
	
End Type
