''*****************************************************************************
''
''
''	Explosion Module
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Explosion.bi"


namespace Explosion

#define MAX_EXPLOSIONS 511



type Explode

public:

	declare constructor()
	declare sub Spawn overload( byref Posi as Vector2D, byref Dire as Vector2D, byval ID as integer )
	declare sub Spawn overload( byref Posi as Vector2D, byval ang as integer, byval ID as integer )
	declare sub Spawn overload( byref Posi as Vector2D, byval Dire as Vector2D, byval spd as single, byval ID as integer )
	declare sub Update( byval dt as single )

	
	as integer	Active
	as single	Speed
	as single	TimeActive
	as single	Tima
	as integer	Red
	as integer	Green
	as integer	Blue
	as single	Alpa 
	as integer  Angle	
	as Vector2D Position
	as Vector2D Direction
	
	as integer	DeathID
	
End Type

''*****************************************************************************
''
''
''
''*****************************************************************************
constructor Explode()
	Active = FALSE
	Speed = 400.0f		'' 400 pixels per second
	TimeActive = 1.0f	'' 1 second 
	Tima = 0.0f			'' 
	DeathID = TINY
End Constructor

sub Explode.Spawn overload( byref Posi as Vector2D, byref Dire as Vector2D, byval ID as integer )
	
	Active = TRUE
	Speed = rnd * (100 + int(ID) * 100) + Rnd * ( 100 + int(ID) * 100)	'' 100 pixels per second
	Tima = 0.0		 
	Angle = RAD2DEG(UtilVector2D.Angle( Dire ))
	Position = Posi
	Direction = Dire
	
	DeathID = ID
	TimeActive = 0.3 + RND *( 2 )	 
	
	select case( DeathID )
		case TINY:
			Red = 200
			Green = 200
			Blue = 0
			Alpa = 255
		case MEDIUM:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case LARGE:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case else
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
	end select
		
End Sub

sub Explode.Spawn overload( byref Posi as Vector2D, byval ang as integer, byval ID as integer )
	
	Active = TRUE
	Speed = rnd * (100 + int(ID) * 100) + Rnd * ( 100 + int(ID) * 100)	'' 100 pixels per second
	Tima = 0.0		 
	
	Position = Posi
	Angle = ang
	Direction = Type<Vector2D>( cos(DEG2RAD(Angle)), sin(DEG2RAD(Angle)) )
	
	DeathID = ID
	TimeActive = 0.3 + RND * ( 2 )	 
	
	select case( DeathID )
		case TINY:
			Red = 200
			Green = 200
			Blue = 0
			Alpa = 255
		case MEDIUM:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case LARGE:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case else
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
	end select
		
End Sub

sub Explode.Spawn overload( byref Posi as Vector2D, byval Dire as Vector2D, byval spd as single, byval ID as integer )
	
	Active = TRUE
	Speed = 0	'' 100 pixels per second
	Tima = 0.0		 
	
	Position = Posi
	Direction = Dire
	Angle = 0
	
	DeathID = ID
	TimeActive = 0.3 + RND * ( 2 )	 
	
	select case( DeathID )
		case TINY:
			Red = 200
			Green = 200
			Blue = 0
			Alpa = 255
		case MEDIUM:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case LARGE:
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
		case else
			Red = rnd * 255
			Green = rnd * 255
			Blue = rnd * 255
			Alpa = 255
	end select

end sub

sub Explode.Update( byval dt as single )
	
	Position += (Direction * Speed) * dt
	
	Alpa -= 150 * dt
	if( Alpa < 0 ) then Alpa = 0
	
	
	Tima += 1 * dt
	
	if( Tima >= TimeActive ) then
		Active = FALSE
	endif
	
end sub

''*****************************************************************************
''
''
''
''*****************************************************************************	
dim as Explode Explosions( MAX_EXPLOSIONS )
dim as GL2D.IMAGE ptr FlareImage
dim as integer NumExplosions = 0
''*****************************************************************************
''
''
''
''*****************************************************************************	
sub Init( byref filename as string )

	NumExplosions = 0
	FlareImage = GL2D.LoadBmpToGLsprite( filename, GL_LINEAR )
	
End Sub

sub Spawn overload( byref Posi as Vector2D, byref Dire as Vector2D, byval ID as integer )
	
	for i as integer = 0 to MAX_EXPLOSIONS
		if( not Explosions(i).Active ) then
			Explosions(i).Spawn( Posi, Dire, ID )
			exit for
		endif
	next i
	
End Sub

sub Spawn overload( byref Posi as Vector2D, byval ang as integer, byval ID as integer )
	
	for i as integer = 0 to MAX_EXPLOSIONS
		if( not Explosions(i).Active ) then
			Explosions(i).Spawn( Posi, ang, ID )
			exit for
		endif
	next i
	
End Sub

sub Spawn overload( byref Posi as Vector2D, byval NumExplosions as integer )
	
	for j as integer = 0 to NumExplosions - 1
		for i as integer = 0 to MAX_EXPLOSIONS
			if( not Explosions(i).Active ) then
				dim as integer ang = rnd * 360
				dim as integer ID = int(rnd * 3)
				Explosions(i).Spawn( Posi, ang, ID )
				exit for
			endif
		next i
	next j
	
End Sub

sub Spawn overload( byref Posi as Vector2D, byval Dire as Vector2D, byval spd as single, byval ID as integer )
	
	for i as integer = 0 to MAX_EXPLOSIONS
		if( not Explosions(i).Active ) then
			Explosions(i).Spawn( Posi, Dire, spd, ID )
			exit for
		endif
	next i
	
End Sub

sub Update( byval dt as single )
	
	NumExplosions = 0
	for i as integer = 0 to MAX_EXPLOSIONS
		if( Explosions(i).Active ) then
			Explosions(i).Update( dt )
			numExplosions += 1
		endif
	next i
	
End Sub
	
sub KillAll()
	
	for i as integer = 0 to MAX_EXPLOSIONS
		if( Explosions(i).Active ) then
			Explosions(i).Active = FALSE
		endif
	next i
	NumExplosions = 0
	
End Sub

sub DrawAll()
	
	dim as Vector2D D
	
	for i as integer = 0 to MAX_EXPLOSIONS
		if( Explosions(i).Active ) then
			glColor4ub( Explosions(i).Red, Explosions(i).Green, Explosions(i).Blue, Explosions(i).Alpa )
			select case( Explosions(i).DeathID )
				case TINY:
					GL2D.SpriteRotateScaleXY( Explosions(i).Position.x,_
											  Explosions(i).Position.y,_
											  Explosions(i).Angle,_
											  0.5,_
											  0.1,_
											  GL2D.FLIP_NONE,_
											  FlareImage )
				case MEDIUM:
					D = Vector2D( Explosions(i).Position + Explosions(i).Direction * 5) 
					GL2D.LineGlow( Explosions(i).Position.x, Explosions(i).Position.y, D.x, D.y, 64 , GL2D_RGBA(Explosions(i).Red, Explosions(i).Green, Explosions(i).Blue,Explosions(i).Alpa ) )
									
				case LARGE:
					GL2D.SpriteRotateScaleXY( Explosions(i).Position.x,_
											  Explosions(i).Position.y,_
											  Explosions(i).Angle,_
											  1.5,_
											  0.3,_
											  GL2D.FLIP_NONE,_
											  FlareImage )
			end select
		endif
	next i
	
	glColor4ub( 255, 255, 255, 255 )
			
End Sub


sub Release()
	
	GL2D.DestroyImage( FlareImage )

End Sub

End Namespace
