''*****************************************************************************
''
''
''	Polygon class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Polygon.bi"

''*****************************************************************************
''
''
''
''*****************************************************************************
constructor Polygon()	
	NumVertices = 0
end constructor

constructor Polygon( byval NumVerts as integer, byval Radius as single, byval ScaleX as single = 1, byval ScaleY as single = 1 )	
	
	if( VerticesLocal  ) then delete[] VerticesLocal
	if( VerticesScreen ) then delete[] VerticesScreen
	if( NormalsLocal   ) then delete[] NormalsLocal
	if( NormalsScreen  ) then delete[] NormalsScreen
	
	NumVertices = NumVerts
	assert(NumVertices)
	VerticesLocal  = new Vector2D[NumVerts]
	VerticesScreen = new Vector2D[NumVerts]
	NormalsLocal   = new Vector2D[NumVerts]
	NormalsScreen  = new Vector2D[NumVerts]
	
	assert(VerticesLocal)
	assert(VerticesScreen)
	assert(NormalsLocal)
	assert(NormalsScreen)
	
	'' Verts
	for i as integer = 0 to NumVertices - 1
		dim as single angle = DEG2RAD( 360.0 / NumVertices * i )
		dim as Vector2D v = type<Vector2D>( cos(angle) * Radius * ScaleX, sin(angle) * Radius * ScaleY )
		VerticesLocal[i] = v
		VerticesScreen[i] = v
	next i
	
	
	'' Normals
	for i as integer = 0 to NumVertices - 1
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
		v = v.GetNormal()
		v.Normalize()
		NormalsLocal[i] = v
		NormalsScreen[i] = v
	next i
	
end constructor

constructor Polygon( byval NumVerts as integer, Vertices() as Vector2D, byval ScaleX as single = 1, byval ScaleY as single = 1 )	
	
	if( VerticesLocal  ) then delete[] VerticesLocal
	if( VerticesScreen ) then delete[] VerticesScreen
	if( NormalsLocal   ) then delete[] NormalsLocal
	if( NormalsScreen  ) then delete[] NormalsScreen
	
	NumVertices = NumVerts
	assert(NumVertices)
	VerticesLocal  = new Vector2D[NumVerts]
	VerticesScreen = new Vector2D[NumVerts]
	NormalsLocal   = new Vector2D[NumVerts]
	NormalsScreen  = new Vector2D[NumVerts]
	
	assert(VerticesLocal)
	assert(VerticesScreen)
	assert(NormalsLocal)
	assert(NormalsScreen)

		'' Verts
	for i as integer = 0 to NumVertices - 1
		dim as Vector2D v 
		v.x = Vertices(i).x * ScaleX
		v.y = Vertices(i).y * ScaleY
		VerticesLocal[i] = v
		VerticesScreen[i] = v
	next i
	
	
	'' Normals
	for i as integer = 0 to NumVertices - 1
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
		v = v.GetNormal()
		v.Normalize()
		NormalsLocal[i] = v
		NormalsScreen[i] = v
	next i
	
end constructor

destructor Polygon()	
	
	NumVertices = 0
	if( VerticesLocal  ) then delete[] VerticesLocal
	if( VerticesScreen ) then delete[] VerticesScreen
	if( NormalsLocal   ) then delete[] NormalsLocal
	if( NormalsScreen  ) then delete[] NormalsScreen
	
end destructor

''*****************************************************************************
''
''
''
''*****************************************************************************
property Polygon.Vertex( byval Index as integer ) as Vector2D
	property = VerticesScreen[ Index mod NumVertices ]
end property

property Polygon.Normal( byval Index as integer ) as Vector2D
	property = NormalsScreen[ Index mod NumVertices ]
end property

''*****************************************************************************
''
''
''
''*****************************************************************************

sub Polygon.Create( byval NumVerts as integer, byval Radius as single, byval ScaleX as single = 1, byval ScaleY as single = 1 )	
	
	if( VerticesLocal  ) then delete[] VerticesLocal
	if( VerticesScreen ) then delete[] VerticesScreen
	if( NormalsLocal   ) then delete[] NormalsLocal
	if( NormalsScreen  ) then delete[] NormalsScreen
	
	NumVertices = NumVerts
	assert(NumVertices)
	VerticesLocal  = new Vector2D[NumVerts]
	VerticesScreen = new Vector2D[NumVerts]
	NormalsLocal   = new Vector2D[NumVerts]
	NormalsScreen  = new Vector2D[NumVerts]
	
	assert(VerticesLocal)
	assert(VerticesScreen)
	assert(NormalsLocal)
	assert(NormalsScreen)

	'' Verts
	for i as integer = 0 to NumVertices - 1
		dim as single angle = DEG2RAD( 360.0 / NumVertices * i )
		dim as Vector2D v = type<Vector2D>( cos(angle) * Radius * ScaleX, sin(angle) * Radius * ScaleY )
		VerticesLocal[i] = v
		VerticesScreen[i] = v
	next i
	
	'' Normals
	for i as integer = 0 to NumVertices - 1
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
		v = v.GetNormal()
		v.Normalize()
		NormalsLocal[i] = v
		NormalsScreen[i] = v
	next i
	
end sub

sub Polygon.Create( byval NumVerts as integer, Vertices() as Vector2D, byval ScaleX as single = 1, byval ScaleY as single = 1 )	
	
	if( VerticesLocal  ) then delete[] VerticesLocal
	if( VerticesScreen ) then delete[] VerticesScreen
	if( NormalsLocal   ) then delete[] NormalsLocal
	if( NormalsScreen  ) then delete[] NormalsScreen
	
	NumVertices = NumVerts
	assert(NumVertices)
	VerticesLocal  = new Vector2D[NumVerts]
	VerticesScreen = new Vector2D[NumVerts]
	NormalsLocal   = new Vector2D[NumVerts]
	NormalsScreen  = new Vector2D[NumVerts]
	
	assert(VerticesLocal)
	assert(VerticesScreen)
	assert(NormalsLocal)
	assert(NormalsScreen)

	'' Verts
	for i as integer = 0 to NumVertices - 1
		dim as Vector2D v 
		v.x = Vertices(i).x * ScaleX
		v.y = Vertices(i).y * ScaleY
		VerticesLocal[i] = v
		VerticesScreen[i] = v
	next i
	
	
	'' Normals
	for i as integer = 0 to NumVertices - 1
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
		v = v.GetNormal()
		v.Normalize()
		NormalsLocal[i] = v
		NormalsScreen[i] = v
	next i
	
end sub


sub Polygon.Reload( byval NumVerts as integer, Vertices() as Vector2D )
	
	assert(NumVertices)
	'' Verts
	for i as integer = 0 to NumVertices - 1
		VerticesLocal[i] = Vertices(i)
		VerticesScreen[i] = Vertices(i)
	next i
	
	
	'' Normals
	for i as integer = 0 to NumVertices - 1
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
		v = v.GetNormal()
		v.Normalize()
		NormalsLocal[i] = v
		NormalsScreen[i] = v
	next i
	
end sub

sub Polygon.ReloadScreenCoords( byval NumVerts as integer, Vertices() as Vector2D, byval RecalcNormals as integer = FALSE )
	
	assert(NumVertices)
	'' Verts
	for i as integer = 0 to NumVertices - 1
		VerticesScreen[i] = Vertices(i)
	next i
	
	
	'' Normals
	if( RecalcNormals ) then
		
		for i as integer = 0 to NumVertices - 1
			dim as integer j = (i+1) mod NumVertices	
			dim as Vector2D v = type<Vector2D>( VerticesLocal[j] - VerticesLocal[i] )
			v = v.GetNormal()
			v.Normalize()
			NormalsLocal[i] = v
			NormalsScreen[i] = v
		next i
	
	endif
	
end sub

sub Polygon.Translate( byval x as single, byval y as single )
	
	CenterX = x
	CenterY = y
	
	Transformation.Translate( x, y )
	
end sub

sub Polygon.Rotate( byval Degrees as integer)
	
	Transformation.Rotate( Degrees ) 
	
	'' Rotate Normals
	dim as Matrix2D M2
	M2.LoadIndentity()
	M2.LoadRotation( Degrees )
		
	for i as integer = 0 to NumVertices - 1
		dim as Vector2D v = type<Vector2D>( M2.TransformPoint( NormalsLocal[i] ) )
		NormalsScreen[i] =  v
	next i
	
end sub

sub Polygon.Scale( byval x as single, byval y as single )
	Transformation.Scale( x, y )
end sub

sub Polygon.Transform()
	
	for i as integer = 0 to NumVertices - 1
		dim as Vector2D v = type<Vector2D>( Transformation.TransformPoint( VerticesLocal[i] ) )
		VerticesScreen[i] =  v
	next i
	
end sub
	
sub Polygon.Reset()
	Transformation.LoadIndentity()
end sub

function Polygon.Project( byref Axis as Vector2D ) as Projection
	
	dim as Projection Proj
	dim as single min, max
	dim as single p
	
	'' Get initial projection on first vertex
	'' and set min and max values to initial projection value
	p = UtilVector2D.Dot( VerticesScreen[0], Axis)
	min = p
	max = p
	
	'' Loop through all the verts of the poly 
	'' and find out if new projections of vertex
	'' is the new min or max as we need only the 
	'' min and max points to test for 1d intesection
	for i as integer = 1 to NumVertices - 1
		p = UtilVector2D.Dot( VerticesScreen[i], Axis)
		if( p < min ) then min = p
		if( p > max ) then max = p
	next i
	
	'' Copy and return
	Proj.Min = min
	Proj.Max = max
	
	return Proj
	
end function

function Polygon.Collide( byref P2 as Polygon ) as integer
	
	dim as Projection proj1
	dim as Projection proj2
	dim as Vector2D axis
	dim as single d1, d2
	
	'' project all the verts of the poly to each axis (normal)
	'' of the poly we are testing and find out if the projections
	'' overlap (ie: length if proj1 and proj2 are intersecting).
	'' if they are intersecting, there is an axis (line perpendicular 
	'' to the axis tested or the "edge" of the poly where the normal connects)
	'' that separates the two polygons so we do an early out from the function.
	
	
	'' polygon1
	for i as integer = 0 to NumVertices - 1
		
		axis = Normal(i)
		proj1 = Project(axis)
		proj2 = P2.Project(axis)
		d1 = (proj1.Min - proj2.Max)
	    d2 = (proj2.Min - proj1.Max)
	    if( (d1 > 0) or (d2 > 0) ) then return FALSE	 '' there's a separating axis so get out early
	        
	next i
	
	
	'' polygon2
	for i as integer = 0 to P2.NumVertices - 1
		
		axis = P2.Normal(i)
		proj1 = Project(axis)
		proj2 = P2.Project(axis)
		d1 = (proj1.Min - proj2.Max)
	    d2 = (proj2.Min - proj1.Max)
	    if( (d1 > 0) or (d2 > 0) ) then return FALSE	 '' there's a separating axis so get out early
	        
	next i
	
	
	'' No separating axis found so p1 and p2 are colliding
	return true
	
end function


sub Polygon.Draw( byval LineWidth as integer, byval colour as GLuint )
	
	for i as integer = 0 to NumVertices - 1
		
		dim as integer j = (i+1) mod NumVertices	
		dim as integer x1 = VerticesScreen[i].x
		dim as integer y1 = VerticesScreen[i].y
		dim as integer x2 = VerticesScreen[j].x
		dim as integer y2 = VerticesScreen[j].y
		GL2D.LineGlow( x1, y1, x2, y2, LineWidth, Colour )
		
	next i
	
end sub

sub Polygon.DrawNormals( byval Length as single, byval colour as GLuint ) 
	
	for i as integer = 0 to NumVertices - 1
		
		dim as integer j = (i+1) mod NumVertices	
		dim as Vector2D Midpoint = UtilVector2D.MidPoint( VerticesScreen[i], VerticesScreen[j] )
		GL2D.Line( Midpoint.x, Midpoint.y, Midpoint.x + NormalsScreen[i].x * Length, Midpoint.y + NormalsScreen[i].y * Length, Colour )
	
	next i
	
end sub

