''*****************************************************************************
''
''
''	PowerUp class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "PowerUp.bi"


''*****************************************************************************
''
'' PowerUp
''
''*****************************************************************************
constructor PowerUp()

	Active = FALSE	
	
	Body.Create( 8, 20 )
	
	Position = Vector2D( -SCREEN_WIDTH, -SCREEN_HEIGHT )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
	
End Constructor

destructor PowerUp()

End Destructor

''*****************************************************************************
''
''
''
''*****************************************************************************
property PowerUp.IsActive() as integer
	property = Active
End Property

property PowerUp.GetPoly() as Polygon
	property = Body
End Property
	
''*****************************************************************************
''
''
''
''*****************************************************************************
sub PowerUp.Init( byref Posi as Vector2D )

	Active = TRUE
	Speed = ( 100.0f + rnd * 200.0f )
	Angle = rnd * ( 2 * PI )
	Colour = GL2D_RGBA( 255, 255, 255, 255 )
	Delta = 8.0f
	Intensity = 1.0f
	Direction = Vector2D( cos(Angle) * Speed, sin(Angle) * Speed )
	
	Position = Posi
	
end sub		   		

sub PowerUp.Update( byref Fighter as ship, byval dt as single )
	
	Position += Direction * dt
	
	if( Position.x < 0 ) then 
		Position.x = 0
		Direction.x = -Direction.x
	EndIf
	if( Position.x > SCREEN_WIDTH ) then 
		Position.x = SCREEN_WIDTH
		Direction.x = -Direction.x
	EndIf
	if( Position.y < 0 ) then 
		Position.y = 0
		Direction.y = -Direction.y
	EndIf
	if( Position.y > SCREEN_HEIGHT ) then
		Position.y = SCREEN_HEIGHT
		Direction.y = -Direction.y
	EndIf
	
	Angle += Delta * dt
	Intensity = 30 + abs(sin((Angle/5.0f))) * 5
	Body.Reset()
	Body.Rotate( RAD2DEG(Angle) ) 
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Sub

sub PowerUp.Draw()
	
	for i as integer = 0 to 2
		Body.Draw( 8 + (i * Intensity), GL2D_RGBA( 255, 255, 0, 255 ) )
	next i
		
End Sub

''*****************************************************************************
''
''
''
''*****************************************************************************
sub PowerUp.Destroy()

	Kill()
	
End Sub

sub PowerUp.Kill()	
	
	Active = FALSE
	Position = Vector2D( -3600, -3600 )	
	Body.Reset()
	Body.Translate( Position.x, Position.y )
	Body.Transform()
	
End Sub
	
''*****************************************************************************
''
'' PowerUpFactory
''
''*****************************************************************************

constructor PowerUpFactory()

End Constructor

destructor PowerUpFactory()

End Destructor

property PowerUpFactory.GetNumEntities() as Integer
	property = NumEntities
end property 

sub PowerUpFactory.UpdateEntities( byref Fighter as Ship, byval dt as single )
	
	NumEntities = 0
	for i as integer = 0 to MAX_POWERUPS
		if( PowerUps(i).IsActive ) then
			PowerUps(i).Update( Fighter, dt )
			NumEntities += 1
		EndIf
	Next
	
end sub

sub PowerUpFactory.DrawEntities()
	
	for i as integer = 0 to MAX_POWERUPS
		if( PowerUps(i).IsActive ) then
			PowerUps(i).Draw()
		EndIf
	Next
	
end sub

sub PowerUpFactory.KillAllEntities()
	
	for i as integer = 0 to MAX_POWERUPS
		if( PowerUps(i).IsActive ) then
			PowerUps(i).Kill()
		EndIf
	Next
	
end sub

sub PowerUpFactory.HandleCollisions( byref Fighter as Ship )
	
	for i as integer = 0 to MAX_POWERUPS
		
		if( PowerUps(i).IsActive ) then
			'' Ship to PowerUp collisions
			if( Fighter.HandleCollision( PowerUps(i).Body) ) then
				Fighter.LevelUp()
				Fighter.AddToScore( 1000 )
				PowerUps(i).Destroy()	
				Sound.PlaySFX( Sound.SFX_POWER_UP )
			end if
		EndIf
		
	Next
	
end sub

sub PowerUpFactory.Spawn( byref Posi as Vector2D )

	
	for i as integer = 0 to MAX_POWERUPS
		if( PowerUps(i).IsActive = FALSE ) then
			PowerUps(i).Init( Posi )
			exit for
		EndIf
	Next
	
end sub
