''*****************************************************************************
''
''
''	Ship class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "Ship.bi"


'' Map 2 buttons for firing (lazy me)
const as integer JOY_FIRE = JOY_KEY_1 or JOY_KEY_2 

const as single FRICTION = 0.0026f * 540
const as single ACCEL = 1.52f * 240

''*****************************************************************************
''
''
''
''*****************************************************************************
constructor Ship()

	const as integer size = 7
	
	dim as Vector2D Verts(2)
	
	'' Set up vertices of MainBody
	Verts(0) = type<Vector2D>(  size * 3,     0 * 2   )
	Verts(1) = type<Vector2D>( -size * 2,  size * 1.5 )
	Verts(2) = type<Vector2D>( -size * 2, -size * 1.5 )
	BodyMain.Create( 3, Verts() )
	
	'' Set up vertices of LeftBody
	Verts(0) = type<Vector2D>(  size * 1,     0 - size * 1.5 )
	Verts(1) = type<Vector2D>( -size * 2, -size - size * 2   )
	Verts(2) = type<Vector2D>( -size * 4,  size - size * 2   )
	BodyLeft.Create( 3, Verts() )
	
	'' Set up vertices of RightBody
	Verts(0) = type<Vector2D>(  size * 1,     0 + size * 1.5 )
	Verts(1) = type<Vector2D>( -size * 2,  size + size * 2   )
	Verts(2) = type<Vector2D>( -size * 4, -size + size * 2   )
	BodyRight.Create( 3, Verts() )
	
	'' Shield
	Shield.Create( 16, 40 )
	
	'' Collision Poly
	Verts(0) = type<Vector2D>(  size * 1.5,     0 * 2   )
	Verts(1) = type<Vector2D>( -size * 1.5,  size * 1.0 )
	Verts(2) = type<Vector2D>( -size * 1.5, -size * 1.0 )
	BodyCollision.Create( 3, Verts() )

	
end constructor

destructor Ship()

end destructor


''*****************************************************************************
''
''
''
''*****************************************************************************
property Ship.IsActive() as integer
	property = Active
end property

property Ship.IsInvincible() as integer
	property = Invincible
end property

property Ship.GetPosition() as Vector2D 
	property = Position
end property
property Ship.SetPosition( byval v as Vector2D ) 
	Position = v
end property

property Ship.GetDirection() as Vector2D
	property = Direction
end property

property Ship.GetAngle() as integer
	property = Angle
end property

property Ship.SetAngle( byval a as integer )
	Angle = a
end property

property Ship.GetScore() as integer
	property = Score
end property

property Ship.GetLives() as integer
	property = Lives
end property

property Ship.GetLevel() as integer
	property = Level
end property

property Ship.GetControlMode() as integer
	property = ControlMode
end property

property Ship.SetControlMode( byval v as MODES )
	ControlMode = v
End Property

property Ship.SetAutoFire( byval v as integer )
	AutoFire = v
End Property
	
''*****************************************************************************
''
''
''
''*****************************************************************************
sub Ship.AddToScore( byval value as integer )
	Score += value
end sub

sub Ship.LevelUp()
	Level += 1
end sub

sub Ship.Initialize()
	
	Frame = 0
	Counter = 0
	Active = true
	Invincible = true
	InvincibleTimer = 0.0f
	
	Position = type<Vector2D>( 320, 500 )
	OldPosition = Position
	Direction = type<Vector2D>( 0, -1 )
	Angle = 270
	Speed = 0   
	SideSpeed = 0
	
	ShotSpawnDelay = 0
	MaxSpawnDelay = 1/10.0f  '' 10 shots per second
	
	Level = 0
	Lives = 3
	Score = 0
	State = STATE_SPAWN
	ControlMode = MODE_NORMAL
	AutoFire = TRUE
		
	Reset()
	Translate( -100, -100 )
	Transform()

	Shield.Reset()
	Shield.Translate( -100, -100)
	Shield.Transform()
	
	ResetJets()
	JetUpdateCoolDown = FIXED_TIME_STEP
	
	for i as integer = 0 to MAX_SHOTS
		Shots(i).Kill()
	next i
	
	
end sub

sub Ship.ReSpawn()
	
	Frame = 0
	Counter = 0
	Active = true
	
	Invincible = true
	InvincibleTimer = 0.0f
	
	Position = type<Vector2D>( 320, 500 )
	OldPosition = Position
	Direction = type<Vector2D>( 0, -1 )
	Angle = 270
	Speed = 0    
	SideSpeed = 0
	
	ShotSpawnDelay = 0
	MaxSpawnDelay = 1/10.0f  '' 10 shots per second
	
	Level = 0
	
	State = STATE_SPAWN
	
	Reset()
	Translate( -100, -100 )
	Transform()

	Shield.Reset()
	Shield.Translate( -100, -100)
	Shield.Transform()
	
	ResetJets()
	JetUpdateCoolDown = FIXED_TIME_STEP
	
	Sound.PlaySFX( Sound.SFX_ENTER )
	
end sub

sub Ship.Update( byref Rat as Mouse, byref Keys as Keyboard, byref Joy as Joystick, byval dt as single, byval SecsElapsed as single )
	
	Frame += 1
	Counter += 1 * dt
	OldPosition = Position
	
	'Level = 3
	select case State
		case STATE_NORMAL:
			if( ControlMode = MODE_NORMAL ) then
				ActionNormal( Keys, Joy, dt )
			else
				ActionNormalMouse( Rat, Keys, Joy, dt )
			endif
		case STATE_SPAWN:
			ActionSpawn( dt )
		case STATE_DIE:
			ActionDie( dt )
	End Select
	
	UpdateShots( dt )
	JetUpdateCoolDown -= dt
	if( JetUpdateCoolDown <= dt ) then
		UpdateJets()
		JetUpdateCoolDown = FIXED_TIME_STEP
	endif

	
End Sub

sub Ship.ActionNormalMouse( byref Rat as Mouse, byref Keys as Keyboard, byref Joy as Joystick, byval dt as single )
	
	dim as single Heading = Speed
	dim as single SideHeading = SideSpeed
	
	'' Get vector perpendicular to direction
	dim as Vector2D Side  = type<Vector2D>( cos(DEG2RAD(Angle)), sin(DEG2RAD(Angle)) )
	
	Side = Side.GetNormal()
	Side.Normalize()
	
	if( Keys.Held( FB.SC_W ) or Keys.Held( FB.SC_UP ) ) or Joy.Up() then Speed += ACCEL * dt
	if( Keys.Held( FB.SC_S ) or Keys.Held( FB.SC_DOWN ) ) or Joy.Down() then Speed -= ACCEL * dt
	if( Keys.Held( FB.SC_A ) or Keys.Held( FB.SC_LEFT ) ) or Joy.Left() then SideSpeed += ACCEL * dt
	if( Keys.Held( FB.SC_D ) or Keys.Held( FB.SC_RIGHT ) ) or Joy.Right() then SideSpeed -= ACCEL * dt
	
	''Apply friction
    Speed -= Speed * FRICTION * dt
	SideSpeed -= SideSpeed * FRICTION * dt
	
	dim as Vector2D MousePos = type<Vector2D>( Rat.GetX, Rat.GetY )
	'' Limit the ship not to go over the mouse position 
	if( UtilVector2D.distance( Position, MousePos ) < 60 ) then
		Direction = type<Vector2D>( cos(DEG2RAD(Angle)) * Heading, sin(DEG2RAD(Angle)) * Heading )
		SideDirection =  Side * SideHeading
	else
		dim as single a = UtilVector2D.AngleBetween(Position, MousePos)
		Angle = RAD2DEG( a )
		Direction = type<Vector2D>( cos(a) * Heading, sin(a) * Heading )
		SideDirection =  Side * SideHeading
	endif
	
	
	Position += Direction * dt + SideDirection * dt
	
	if( Position.x < 0 ) then Position.x = 0
	if( Position.x > SCREEN_WIDTH ) then Position.x = SCREEN_WIDTH
	if( Position.y < 0 ) then Position.y = 0
	if( Position.y > SCREEN_HEIGHT ) then Position.y = SCREEN_HEIGHT
	
	'' Transform ship polygon
	Reset()
	Rotate( Angle )
	Translate( Position.x, Position.y )
	Transform()
	

	if( Keys.Held( FB.SC_SPACE ) or Keys.Held( FB.SC_Z ) or Keys.Held( FB.SC_X ) or Joy.KeyHeld( JOY_FIRE ) or Rat.ButtonHeld( MOUSE_BUTTON_LEFT or MOUSE_BUTTON_RIGHT or MOUSE_BUTTON_MIDDLE )  or AutoFire ) then
		SpawnShots( dt, Side )
	end if 
	
	
	if( Invincible ) then
		InvincibleTimer += 1 * dt
		if( InvincibleTimer > 5.0f ) then
			Invincible = false
			InvincibleTimer = 0
		endif
		Shield.Reset()
		Shield.Scale( (5.0f - InvincibleTimer)/5.0f, (5.0f - InvincibleTimer)/5.0f )
		Shield.Rotate( RAD2DEG(InvincibleTimer * 8) )
		Shield.Translate( Position.x, Position.y )
		Shield.Transform()
	endif
	
end sub

sub Ship.ActionNormal( byref Keys as Keyboard, byref Joy as Joystick, byval dt as single )

	dim as single Heading = Speed
	dim as single SideHeading = SideSpeed
	
	'' Get vector perpendicular to direction
	dim as Vector2D Side  = type<Vector2D>( cos(DEG2RAD(Angle)), sin(DEG2RAD(Angle)) )
	
	Side = Side.GetNormal()
	Side.Normalize()
	
	if( Keys.Held( FB.SC_W ) or Keys.Held( FB.SC_UP )  or Joy.Up() ) then Speed += ACCEL * dt
	if( Keys.Held( FB.SC_S ) or Keys.Held( FB.SC_DOWN )  or Joy.Down() ) then Speed -= ACCEL * dt
	if( Keys.Held( FB.SC_A ) or Keys.Held( FB.SC_LEFT )  or Joy.Left() ) then Angle -= 180 * dt
	if( Keys.Held( FB.SC_D ) or Keys.Held( FB.SC_RIGHT ) or Joy.Right() ) then Angle += 180 * dt
	
	
	''Apply friction
    Speed -= Speed * FRICTION * dt
	SideSpeed -= SideSpeed * FRICTION * dt
	
	
	Direction = type<Vector2D>( cos(DEG2RAD(Angle)) * Heading, sin(DEG2RAD(Angle)) * Heading )
	
	Position += Direction * dt 
	
	if( Position.x < 0 ) then Position.x = 0
	if( Position.x > SCREEN_WIDTH ) then Position.x = SCREEN_WIDTH
	if( Position.y < 0 ) then Position.y = 0
	if( Position.y > SCREEN_HEIGHT ) then Position.y = SCREEN_HEIGHT
	
	'' Transform ship polygon
	Reset()
	Rotate( Angle )
	Translate( Position.x, Position.y )
	Transform()
	
	'Level = 3
	if( Keys.Held( FB.SC_SPACE ) or Keys.Held( FB.SC_Z ) or Keys.Held( FB.SC_X ) or Joy.KeyHeld( JOY_FIRE )  or AutoFire ) then
		SpawnShots( dt, Side )
	endif
	
	if( Invincible ) then
		InvincibleTimer += 1 * dt
		if( InvincibleTimer > 5.0f ) then
			Invincible = false
			InvincibleTimer = 0
		endif
		Shield.Reset()
		Shield.Scale( (5.0f - InvincibleTimer)/5.0f, (5.0f - InvincibleTimer)/5.0f )
		Shield.Rotate( RAD2DEG(InvincibleTimer * 8) )
		Shield.Translate( Position.x, Position.y )
		Shield.Transform()
	endif
	
	
end sub

sub Ship.ActionSpawn( byval dt as single )
	
	'' Move the player from bottom to middle of screen
	dim as Vector2D D = type<Vector2D>( cos(DEG2RAD(Angle)), sin(DEG2RAD(Angle)) ) * 300
	
	Position += D * dt 
	
	'' Transform ship polygon
	Reset()
	Rotate( Angle )
	Translate( Position.x, Position.y )
	Transform()
	
	
	Shield.Reset()
	Shield.Scale( (5.0f - InvincibleTimer)/5.0f, (5.0f - InvincibleTimer)/5.0f )
	Shield.Rotate( RAD2DEG( Position.y * -4 ) )
	Shield.Translate( Position.x, Position.y )
	Shield.Transform()

	'' Change state when we hit middle of screen
	if( Position.y <= SCREEN_HEIGHT\2 ) then 
		Counter = 0
		State = STATE_NORMAL
	EndIf
	
End Sub

sub Ship.ActionDie( byval dt as single )
	
	static as integer FirstTime = TRUE
	static as single x1 = 0
	static as single y1 = 0
	static as single x2 = 0
	static as single y2 = 0
	
	if( FirstTime ) then
		x1 = -2 + rnd * (4)
		y1 = -2 + rnd * (4)
		x2 = -2 + rnd * (4)
		y2 = -2 + rnd * (4)
		FirstTime = FALSE
	EndIf
	
	Reset()
	BodyMain.Rotate( Counter * 1230 )
	BodyLeft.Rotate( -Counter * 1230 )
	BodyRight.Rotate( Counter * 1230 )
	BodyCollision.Rotate( -Counter * 1230 )
	
	Translate( Position.x, Position.y )
	
	BodyMain.Translate( x1 * Counter * 150, y1 * Counter * 150 )
	BodyLeft.Translate( y1 * Counter * 200, x1 * Counter * 200 )
	BodyRight.Translate( x2 * Counter * 300, y2 * Counter * 300 )
	BodyCollision.Translate( y2 * Counter * 100, x2 * Counter * 100 )
	
	Transform()
	if( Lives > 0 ) then	
		if( Counter > 1.5 ) then 
			Counter = 0
			State = STATE_SPAWN
			ReSpawn()
			x1 = -2 + rnd * (4)
			y1 = -2 + rnd * (4)
			x2 = -2 + rnd * (4)
			y2 = -2 + rnd * (4)
		EndIf
	endif
	
End Sub

sub Ship.Draw( byval Intensity as single = 1.0 )
	
	const as integer LineWidth = 10
	
	select case State
		case STATE_NORMAL:
			
			for i as integer = 0 to 4
				dim as integer a = (i / 5.0f) * 128
				BodyMain.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 128 + a ) )
				BodyLeft.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyRight.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyCollision.Draw( 6 + ( i * Intensity ), GL2D_RGBA( 64, 128, 255, 128 + a ) )
			next i
			
			''DrawShots()
			if( Invincible ) then
				for i as integer = 0 to 4
					dim as integer a = (i / 5.0f) * 128
					Shield.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 128, 255, 64, 128 + a ) )
				next i	
			endif
			DrawBoosters()
	
		case STATE_SPAWN:
			
			for i as integer = 0 to 4
				dim as integer a = (i / 5.0f) * 128
				BodyMain.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 128 + a ) )
				BodyLeft.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyRight.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyCollision.Draw( 6 + ( i * Intensity ), GL2D_RGBA( 64, 128, 255, 128 + a ) )
			next i
			for i as integer = 0 to 4
				dim as integer a = (i / 5.0f) * 128
				Shield.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 128, 255, 64, 128 + a ) )
			next i	
			
			DrawBoosters()
	
		case STATE_DIE:
			
			for i as integer = 0 to 4
				dim as integer a = (i / 5.0f) * 128
				BodyMain.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 255, 64, 64, 128 + a ) )
				BodyLeft.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyRight.Draw( LineWidth - ( i * Intensity ), GL2D_RGBA( 200, 128, 0, 128 + a ) )
				BodyCollision.Draw( 6 + ( i * Intensity ), GL2D_RGBA( 64, 128, 255, 128 + a ) )
			next i
			
	End Select
	
	DrawJets( GL2D_RGBA( 200, 128, 0, 128 + 255 ) )
	
	DrawShots()
		
end sub

sub Ship.SpawnShot( byref Position as Vector2D, byval Angle as integer, byval Col as integer, byref SpeedUp as Vector2D = type<Vector2D>(0,0), byval length as single = 20 )

	for i as integer = 0 to MAX_SHOTS
		if( not Shots(i).IsActive ) then
			Shots(i).Init( Position, Angle, Col, Vector2D(0,0), length )
			exit for
		endif
	next i
	
end sub

sub Ship.UpdateShots( byval dt as single )
	
	for i as integer = 0 to MAX_SHOTS
		if( Shots(i).IsActive ) then
			Shots(i).Update(dt)
		endif
	next i
	
end sub

sub Ship.DrawShots()
	
	for i as integer = 0 to MAX_SHOTS
		if( Shots(i).IsActive ) then
			Shots(i).Draw()
		endif
	next i
	
end sub

function Ship.HandleCollision( byref Poly as Polygon ) as integer
	
	if( Active ) then
		if( BodyCollision.Collide( Poly ) ) then
			return TRUE
		endif
	endif

	return FALSE
	
end function

function Ship.HandleShotCollisions( byref Poly as Polygon ) as integer
	
	if( Active ) then
		for i as integer = 0 to MAX_SHOTS
			if( Shots(i).IsActive ) then
				if( Shots(i).Collide( Poly ) ) then
					Explosion.Spawn(Shots(i).GetPosition + Shots(i).GetHeading * 20, rnd * 360, Explosion.TINY )
					Shots(i).Kill()
					Sound.PlaySFX( Sound.SFX_ENEMY_HIT )
					return Shots(i).GetPower
				endif
			endif
		next i
	endif
	return 0
	
end function

sub Ship.Destroy()
	
	State = STATE_DIE
	Counter = 0
		
	Explosion.Spawn( Position, 200 )
	Sound.PlaySFX( Sound.SFX_EXPLODE_03 )
	Kill()
	
end sub

sub Ship.Kill()
	
	Active = FALSE
	'Position = type<Vector2D>( -SCREEN_WIDTH, -SCREEN_HEIGHT )	
	'Reset()
	'Translate( Position.x, Position.y )
	'Transform()
	
	Lives -= 1
	
end sub


''*****************************************************************************
''
''
''
''*****************************************************************************
sub Ship.Translate( byval x as single, byval y as single )
	
	BodyMain.Translate( x, y )
	BodyLeft.Translate( x, y )
	BodyRight.Translate( x, y )
	BodyCollision.Translate( x, y )
	
end sub

sub Ship.Rotate( byval Degrees as integer)
	
	BodyMain.Rotate( Degrees )
	BodyLeft.Rotate( Degrees )
	BodyRight.Rotate( Degrees )
	BodyCollision.Rotate( Degrees )
	
end sub

sub Ship.Scale( byval x as single, byval y as single )
	
	BodyMain.Scale( x, y )
	BodyLeft.Scale( x, y )
	BodyRight.Scale( x, y )
	BodyCollision.Scale( x, y )
	
end sub

sub Ship.Transform()
	
	BodyMain.Transform()
	BodyLeft.Transform()
	BodyRight.Transform()
	BodyCollision.Transform()
	
end sub

sub Ship.Reset()
	
	BodyMain.Reset()
	BodyLeft.Reset()
	BodyRight.Reset()
	BodyCollision.Reset()
	
end sub

sub Ship.SpawnShots( byval dt as single, byref Side as Vector2D )
	
	'' Autofire!!!
	ShotSpawnDelay -= 1 * dt
	if( ShotSpawnDelay < 0 ) then
		if( Level < 3 ) then 
			ShotSpawnDelay = MaxSpawnDelay + 0.03
		else
			ShotSpawnDelay = MaxSpawnDelay
		EndIf
		'' Center
		SpawnShot( Position + Vector2D( Side.GetNormal() ) * -20, Angle, GL2D_RGBA( 255, 0, 0, 255 ) )
		if( Level > 0 ) then
			'' Left
			SpawnShot( Position + type<Vector2D>( Side ) * -20, Angle - (10 * sin(timer * 4)), GL2D_RGBA( 255, 128, 0, 255 ) )
			'' Right
			SpawnShot( Position + type<Vector2D>( Side ) *  20, Angle + (10 * sin(timer * 4)), GL2D_RGBA( 255, 128, 0, 255 ) )
		endif		
		if( Level > 1 ) then
			'' Center Left
			SpawnShot( Position + type<Vector2D>( Side ) * -10, Angle + (10 * sin(timer * 4)), GL2D_RGBA( 0, 255, 255, 255 ) )
			'' Ceneter Right
			SpawnShot( Position + type<Vector2D>( Side ) *  10, Angle - (10 * sin(timer * 4)), GL2D_RGBA( 0, 255, 255, 255 ) )
		endif
		
		Sound.PlaySFX( Sound.SFX_SHOT )
		
	endif
	

End Sub

sub Ship.ResetJets()
	
	for i as integer = 0 to JET_MAX_SEGMENT
        Jets(i)= Position
    next i
	
End Sub

sub Ship.UpdateJets()
	
	dim as single a = DEG2RAD(Angle + PI/2)
	dim as Vector2D Offset = -Vector2D( cos(a), sin(a) )
	Jets(0) = Position + (Offset * 20)
	'' Reverse copy
    for i as integer = ( JET_MAX_SEGMENT ) to 1 step -1
        Jets(i) = Jets(i-1)
    next i
	
End Sub

sub Ship.DrawJets( byval Colour as integer )
	
	for i as integer = ( JET_MAX_SEGMENT ) to 1 step -1
        dim as Vector2D P1 = Jets(i)
    	dim as Vector2D P2 = Jets(i-1)
    	dim as integer strength1 = ((JET_MAX_SEGMENT-i-1)/JET_MAX_SEGMENT) * 255
		dim as integer strength2 = ((JET_MAX_SEGMENT-i)/JET_MAX_SEGMENT) * 255
		GL2D.LineGlow( P1.x, P1.y, P2.x, P2.y, 15, GL2D_RGBA(strength2,0,strength1,strength1))
    next i
	
End Sub

sub Ship.DrawBoosters()
	
	dim as single a = DEG2RAD(Angle - (90/3))
	dim as Vector2D OffsetR = Vector2D( cos(a), sin(a) ) * 27
	
	a = DEG2RAD(Angle + (90/3))
	dim as Vector2D OffsetL = Vector2D( cos(a), sin(a) ) * 27
	
	a = DEG2RAD(Angle)
	dim as Vector2D OffsetC = Vector2D( cos(a), sin(a) ) * 15
	
	OffsetR = Position - OffsetR
	OffsetL = Position - OffsetL
	OffsetC = Position - OffsetC
	
	a = Angle
	
	dim as single leng = 5 + abs(sin(Counter * 20 )) * 5
	for i as integer = 0 to 5
		dim as single Intensity = 5 + (sin(Counter * (i+1) * 10 )) * 5
		GL2D.LineGlow( OffsetR.x, OffsetR.y, OffsetR.x + cos( DEG2RAD(a - 25)  ) * -Leng, OffsetR.y + sin( DEG2RAD(a - 25) ) * -Leng, 2 + (i * Intensity), GL2D_RGBA( 255, 255- i * Intensity * 60, i * Intensity * 20, 255 ) )
		GL2D.LineGlow( OffsetL.x, OffsetL.y, OffsetL.x + cos( DEG2RAD(a + 25)  ) * -Leng, OffsetL.y + sin( DEG2RAD(a + 25) ) * -Leng, 2 + (i * Intensity), GL2D_RGBA( 255, 255- i * Intensity * 60, i * Intensity * 20, 255 ) )
		GL2D.LineGlow( OffsetC.x, OffsetC.y, OffsetC.x + cos( DEG2RAD(a)       ) * -Leng, OffsetC.y + sin( DEG2RAD(a)      ) * -Leng, 2 + (i * Intensity), GL2D_RGBA( Intensity * 60, 255 , 255, 255 ) )
	next i
			 
End Sub
		