''*****************************************************************************
''
''
''	Ship class
''	Richard Eric M. Lope
''	http://rel.phatcode.net
''
''	
''
''*****************************************************************************

#include once "FBGFX.bi"
#include once "FBGL2D7.bi"

#include once "Vector2D.bi"
#include once "Matrix2D.bi"
#include once "Polygon.bi"
#include once "Keyboard.bi"
#include once "Mouse.bi"
#include once "Joystick.bi"
#include once "Sound.bi"
#include once "Explosion.bi"

#include once "Globals.bi"
#include once "Shot.bi"

#ifndef FALSE
	const as integer FALSE = 0
	const as integer TRUE = NOT FALSE
#endif

#define MAX_SHOTS 127
#define JET_MAX_SEGMENT 63

''*****************************************************************************
''
''
''
''*****************************************************************************

type Ship
	
public:	
	enum STATES
		STATE_NORMAL = 0,
		STATE_SPAWN,
		STATE_DIE
	end enum
	
	enum MODES
		MODE_NORMAL = 0,
		MODE_EXPERIMENTAL
	end enum
	
	'' constructors and destructors
	declare constructor()	
	declare destructor()	
	
	
	declare property IsActive() as integer
	declare property IsInvincible() as integer
	declare property GetPosition() as Vector2D 
	declare property SetPosition(byval v as Vector2D) 
	declare property GetDirection() as Vector2D
	declare property GetAngle() as integer
	declare property SetAngle( byval a as integer )
	declare property GetScore() as integer
	declare property GetLives() as integer
	declare property GetLevel() as integer
	declare property GetControlMode() as integer
	declare property SetControlMode( byval v as MODES )
	declare property SetAutoFire( byval v as integer )
	
	declare sub AddToScore( byval value as integer )
	declare sub LevelUp()
	
	declare sub Initialize()
	declare sub ReSpawn()
	declare sub Update( byref Rat as Mouse, byref Keys as Keyboard, byref Joy as Joystick, byval dt as single, byval SecsElapsed as single )
	declare sub ActionNormalMouse( byref Rat as Mouse, byref Keys as Keyboard, byref Joy as Joystick, byval dt as single )
	declare sub ActionNormal( byref Keys as Keyboard, byref Joy as Joystick, byval dt as single )
	declare sub ActionSpawn( byval dt as single )
	declare sub ActionDie( byval dt as single )
	declare sub Draw( byval Intensity as single = 1.0 )
	declare sub SpawnShot( byref Position as Vector2D, byval Angle as integer, byval Col as integer, byref SpeedUp as Vector2D = type<Vector2D>(0,0), byval leng as single = 20 )
	declare sub UpdateShots( byval dt as single )
	declare sub DrawShots()
	declare function HandleCollision( byref Poly as Polygon ) as integer
	declare function HandleShotCollisions( byref Poly as Polygon ) as integer
	declare sub Destroy()
	declare sub Kill()
	
	declare sub Translate( byval x as single, byval y as single )
	declare sub Rotate( byval Degrees as integer)
	declare sub Scale( byval x as single, byval y as single )
	declare sub Reset()
	declare sub Transform()
	
	
	
private:
	
	
	declare sub SpawnShots( byval dt as single, byref Side as Vector2D )
	declare sub ResetJets()
	declare sub UpdateJets()
	declare sub DrawJets( byval Colour as integer )
	declare sub DrawBoosters()
	
	as integer  Frame
	as single	Counter
	as integer 	Active
	as integer 	Invincible
	as single 	InvincibleTimer
	as integer 	Score
	as integer 	Level
	as integer 	Lives
	
	as Vector2D	Position
	as Vector2D	OldPosition
	as Vector2D	Direction
	as Vector2D	SideDirection
	as single 	Angle
	as single 	Speed
	as single 	SideSpeed
	
	as Polygon 	BodyMain
	as Polygon 	BodyLeft
	as Polygon 	BodyRight
	
	as Polygon 	Shield
	as Polygon 	BodyCollision
	
	as single	ShotSpawnDelay
	as single	MaxSpawnDelay
	as single   JetUpdateCoolDown
	
	as integer	State
	as integer	ControlMode
	as integer  AutoFire
	'as integer  Moved
	
	as Vector2D Jets(JET_MAX_SEGMENT)
	as Shot Shots(MAX_SHOTS)
	
end type
